@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
  *
  */
 
-
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -78,6 +77,7 @@ extern brightonApp poly800App;
 extern brightonApp bme700App;
 extern brightonApp bmApp;
 extern brightonApp sidApp;
+extern brightonApp sid2App;
 
 #define BRIGHTON_BIT99APP (BRISTOL_SYNTHCOUNT + 1)
 
@@ -101,8 +101,10 @@ int dcTimeout = 500000;
 int sampleRate = 44100;
 int sampleCount = 256;
 int mwt = 50000;
-int activeSense = 1000;
-int activeSensePeriod = 3000;
+int activeSense = 2000;
+int activeSensePeriod = 15000;
+
+static char *BRISTOL_DEF_HOST = "localhost";
 
 extern int brightonMidiInput(bristolMidiMsg *);
 
@@ -118,7 +120,7 @@ extern int brightonMidiInput(bristolMidiMsg *);
 int
 main(int argc, char **argv)
 {
-	int argCount = 1, midiHandle = -1, midiFD = -1, i, j, asc = 0;
+	int argCount = 1, midiHandle = -1, cFD = -1, midiFD = -1, i, j, asc = 0;
 	pthread_t thread;
 	bristolMidiMsg msg;
 
@@ -143,6 +145,7 @@ main(int argc, char **argv)
 	global.synths->notepref = 0;
 	global.synths->notetrig = 0;
 	global.port = -1;
+	global.host = BRISTOL_DEF_HOST;
 
 	synthesisers[BRISTOL_MINI] = &miniApp;
 	synthesisers[BRISTOL_PROPHET] = &prophetApp;
@@ -185,9 +188,17 @@ main(int argc, char **argv)
 	synthesisers[BRISTOL_BME700] = &bme700App;
 	synthesisers[BRISTOL_BASSMAKER] = &bmApp;
 	synthesisers[BRISTOL_SID_M1] = &sidApp;
+	synthesisers[BRISTOL_SID_M2] = &sid2App;
 
 	global.synths->location = 0;
 
+	if ((argc > 1) && (strcmp(argv[1], "-summary") == 0))
+	{
+		printf("%s", summarytext);
+		exit(0);
+	}
+
+
 	if (argc < 2) {
 		printf("You probably prefer to use the startBristol script.\n");
 		exit(1);
@@ -445,6 +456,14 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 
 		if ((strcmp(argv[argCount], "-scale") == 0) && (argCount < argc))
 		{
+			if ((strcmp(argv[argCount + 1], "fs") == 0)
+				|| (strcmp(argv[argCount + 1], "fullscreen") == 0))
+			{
+				aliastype = 5;
+				scale = 100.0;
+				argCount++;
+				continue;
+			}
 			if ((scale = atof(argv[argCount + 1])) > 0)
 			{
 				if ((scale > 1.1) || (scale < 0.9))
@@ -543,8 +562,11 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		if (strcmp(argv[argCount], "-mono") == 0)
 			global.synths->voices = 1;
 
-		if (strcmp(argv[argCount], "-mididbg") == 0)
-			global.synths->flags |= MIDI_DEBUG;
+		if (strcmp(argv[argCount], "-mididbg2") == 0)
+			global.synths->flags |= REQ_MIDI_DEBUG2;
+		if ((strcmp(argv[argCount], "-mididbg") == 0)
+			|| (strcmp(argv[argCount], "-mididbg1") == 0))
+			global.synths->flags |= REQ_MIDI_DEBUG;
 
 		if ((strcmp(argv[argCount], "-midinrp") == 0)
 			|| (strcmp(argv[argCount], "-nrp") == 0))
@@ -582,6 +604,9 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 				global.port = atoi(argv[++argCount]);
 		}
 
+		if (strcmp(argv[argCount], "-host") == 0)
+			global.host = argv[++argCount];
+
 		if (strcmp(argv[argCount], "-lwf") == 0)
 			global.synths->lwf = 1;
 
@@ -768,15 +793,31 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		}
 
 		if ((strcmp(argv[argCount], "-sid") == 0)
-			|| (strcmp(argv[argCount], "-asid") == 0)
-			|| (strcmp(argv[argCount], "-resid") == 0)
-			|| (strcmp(argv[argCount], "-sidney") == 0)
-			|| (strcmp(argv[argCount], "-acid") == 0))
+			|| (strcmp(argv[argCount], "-sidney") == 0))
 		{
 			global.synths->synthtype = BRISTOL_SID_M1;
 			global.synths->voices = 1;
 			global.synths->notepref = 2;
 			global.synths->notetrig = 1;
+			antialias = 0.5;
+			aliastype = 1;
+			opacity = 50;
+		}
+
+		if ((strcmp(argv[argCount], "-sid2") == 0)
+			|| (strcmp(argv[argCount], "-melbourne") == 0)
+			|| (strcmp(argv[argCount], "-canberra") == 0)
+			|| (strcmp(argv[argCount], "-perth") == 0)
+			|| (strcmp(argv[argCount], "-resid") == 0)
+			|| (strcmp(argv[argCount], "-asid") == 0)
+			|| (strcmp(argv[argCount], "-acid") == 0))
+		{
+			global.synths->synthtype = BRISTOL_SID_M2;
+			global.synths->voices = 1;
+			global.synths->notepref = 2;
+			global.synths->notetrig = 1;
+			antialias = 0.5;
+			aliastype = 1;
 			opacity = 50;
 		}
 
@@ -1013,20 +1054,22 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 
 	/*
 	 * Here we should open an ALSA SEQ interface. We want to get an index back
-	 * and pass that to the event handler for selection.
+	 * and pass that to the event handler for selection. This should not really
+	 * be a SEQ interface, it should depend on the flags given to the GUI.
 	 */
 	if (global.libtest != 1)
 	{
 		if ((midiHandle = bristolMidiOpen("brighton",
 			BRISTOL_CONN_SEQ|BRISTOL_DUPLEX,
 			-1, BRISTOL_REQ_NSX, brightonMidiInput, &global)) < 0)
-		{
 			printf("Error opening midi device %s\n", "0.0");
-		}
 
 		midiFD = bristolGetMidiFD(midiHandle);
+//		cFD = bristolMidiDupFD(global.controlfd, BRISTOL_REQ_NSX,
+//			 brightonMidiInput, &global);
+//		cFD = bristolGetMidiFD(cFD);
 
-		printf("opened GUI midi handle: %i, fd %i\n", midiHandle, midiFD);
+		printf("opened GUI midi handles: %i, %i\n", midiFD, cFD);
 	}
 
 	brightonReadConfiguration(global.synths->win,
@@ -1068,8 +1111,8 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		 * this into a separate thread and using a semaphore since the 'busy'
 		 * waiting is ugly.
 		 */
-		if (midiHandle >= 0)
-			bristolMidiDevRead(midiHandle, &msg);
+//		bristolMidiDevRead(cFD, &msg);
+		bristolMidiDevRead(midiFD, &msg);
 
 		if (i == 0)
 			usleep(mwt);
@@ -1151,7 +1194,7 @@ cleanout(void *id)
 	if (id)
 		brightonRemoveInterface(id);
 	cleanupBristol();
-	exit(0);
+	exit(4);
 }
 
 void
