@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -165,6 +165,7 @@ typedef struct BmMem {
 	int lastnote;
 	int lastpage;
 	int laststep;
+	int clearstep;
 	int lastcnote;
 	int cMemPage;
 	int cMemEntry;
@@ -1846,6 +1847,7 @@ static void
 bmStartStop(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
 	brightonEvent event;
+	bmMem *bm = (bmMem *) synth->mem.param;
 
 	if (ssEx) {
 		ssEx = 0;
@@ -1872,11 +1874,23 @@ bmStartStop(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		ssEx = 1;
 		event.value = 0.0;
 		brightonParamChange(synth->win, 4, 44, &event);
+		bm->clearstep = bm->laststep;
+		bm->laststep = 0;
 	} else {
 		/*
 		 * There are two cases here, first press and second press. First press
 		 * will start the sequence, second will pause the sequence.
+		 *
+		 * When it is first press we need to clock the first element in the
+		 * list.
 		 */
+		if (bm->clearstep >= 0)
+		{
+			event.value = BRIGHTON_LED_OFF;
+			brightonParamChange(synth->win, bm->lastpage,
+				4 + bm->clearstep * 8, &event);
+		}
+		bm->clearstep = -1;
 
 		/* Turn off Stop LED */
 		event.value = BRIGHTON_LED_OFF;
@@ -1897,6 +1911,11 @@ bmStartStop(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 		brightonFastTimer(0, 0, 0, BRIGHTON_FT_START, 0);
 
+		event.command = BRIGHTON_FAST_TIMER;
+		event.value = 1;
+		brightonParamChange(synth->win, bm->lastpage,
+			4 + bm->laststep * 8, &event);
+
 		ssEx = 1;
 		event.value = 0.0;
 		brightonParamChange(synth->win, 4, 45, &event);
@@ -1955,7 +1974,7 @@ bmInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = bmMidiSendMsg;
