@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -768,9 +768,12 @@ configureGlobals(guiSynth *synth)
 		bristolMidiSendNRP(global.controlfd, synth->midichannel,
 			BRISTOL_NRP_MNL_VELOC, synth->legatovelocity);
 
-		if (synth->flags & MIDI_DEBUG)
+		if (synth->flags & REQ_MIDI_DEBUG)
 			bristolMidiSendNRP(global.controlfd, synth->midichannel,
 				BRISTOL_NRP_DEBUG, 1);
+		if (synth->flags & REQ_MIDI_DEBUG2)
+			bristolMidiSendNRP(global.controlfd, synth->midichannel,
+				BRISTOL_NRP_DEBUG, 2);
 		if (synth->flags & MIDI_NRP)
 			bristolMidiSendNRP(global.controlfd, synth->midichannel,
 				BRISTOL_NRP_ENABLE_NRP, 1);
@@ -791,9 +794,6 @@ configureGlobals(guiSynth *synth)
 				BRISTOL_NRP_VELOCITY, synth->velocity);
 			bristolMidiSendNRP(global.controlfd, synth->midichannel+1,
 				BRISTOL_NRP_LWF, synth->lwf);
-			if (synth->flags & MIDI_DEBUG)
-				bristolMidiSendNRP(global.controlfd, synth->midichannel+1,
-					BRISTOL_NRP_DEBUG, 1);
 			if (synth->flags & MIDI_NRP)
 				bristolMidiSendNRP(global.controlfd, synth->midichannel+1,
 					BRISTOL_NRP_ENABLE_NRP, 1);
