@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -71,8 +71,9 @@ typedef struct Memory {
 #define MEM_LOADING	0x04
 #define SUPPRESS	0x08
 #define NO_KEYTRACK	0x10
-#define MIDI_DEBUG	0x20
+#define REQ_MIDI_DEBUG	0x20
 #define MIDI_NRP	0x40
+#define REQ_MIDI_DEBUG2	0x80
 
 typedef struct GuiSynth {
 	struct GuiSynth *next, *last;
@@ -119,6 +120,7 @@ typedef struct guiMain {
 	int libtest;
 	int voices;
 	guiSynth *synths;
+	char *host;
 	int port;
 } guimain;
 
