@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -1944,8 +1944,8 @@ obxaCallback(brightonWindow * win, int panel, int index, float value)
 	guiSynth *synth = findSynth(global.synths, win);
 	int sendvalue, sid = 0;
 
-printf("obxaCallback(%i, %i, %f): %x, %1.0f\n", panel, index, value, synth,
-global.synths->mem.param[LAYER_SWITCH]);
+//printf("obxaCallback(%i, %i, %f): %x, %1.0f\n", panel, index, value, synth,
+//global.synths->mem.param[LAYER_SWITCH]);
 
 	if (synth->flags & SUPPRESS)
 		return(0);
