@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -394,7 +394,7 @@ brightonApp pro10App = {
 	pro10destroy,
 	{10, 100, 2, 2, 5, 520, 0, 0},
 	745, 410,
-	7, /* one panel only */
+	7,
 	{
 		{
 			"Pro10",
@@ -1634,7 +1634,8 @@ pro10Init(brightonWindow *win)
 	synth->mem.count = DEVICE_COUNT;
 	synth->mem.active = ACTIVE_DEVS;
 	synth->dispatch = (dispatcher *)
-		brightonmalloc(DEVICE_COUNT * sizeof(dispatcher));
+		brightonmalloc(
+			(DEVICE_COUNT + P10_MOD_COUNT + PMOD_COUNT) * sizeof(dispatcher));
 	dispatch = synth->dispatch;
 
 	synth->second = brightonmalloc(sizeof(guiSynth));
