@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -75,7 +75,10 @@ cleanupBristol()
 #ifdef DEBUG
 	printf("cleanupBrighton()\n");
 #endif
-printf("cleanupBrighton(%i)\n", global.controlfd);
+	printf("cleanupBrighton(%i)\n", global.controlfd);
+
+
+	BAutoRepeat(global.synths->win->display, 1);
 
 	if (global.controlfd >= 0)
 	{
@@ -90,12 +93,14 @@ printf("cleanupBrighton(%i)\n", global.controlfd);
 			}
 			bristolMidiSendMsg(global.controlfd, synth->sid,
 				127, 0, BRISTOL_EXIT_ALGO);
+
 			if ((result = bristolMidiRead(global.controlfd, &msg)) < 0)
 				printf("socket closed\n");
+
 			printf("request acked: %i\n", result);
 
-			if (global.libtest == 0)
-				sleep(1);
+			if (result >= 0)
+				sleep(2);
 		}
 	}
 
@@ -748,12 +753,12 @@ forkengine(guimain *global)
 	flags = BRISTOL_CONN_TCP|BRISTOL_DUPLEX|
 		(global->flags &  BRISTOL_CONN_FORCE);
 
-	if ((global->controlfd = bristolMidiOpen("localhost",
+	if ((global->controlfd = bristolMidiOpen(global->host,
 		flags, global->port, -1, NULL, NULL)) < 0)
 	{
 		printf("opening link to engine: %i\n", global->port);
-		if ((global->controlfd = bristolMidiOpen("localhost",
-			flags, global->port, -1, NULL, NULL)) < 0)
+		if ((global->controlfd = bristolMidiOpen(global->host,
+			flags, global->port, BRISTOL_REQ_SYSEX, NULL, NULL)) < 0)
 			cleanupBristol();
 	} else
 		printf("%s already active\n", BRISTOL_ENGINE);
@@ -777,12 +782,14 @@ initConnection(guimain *global, guiSynth *synth)
 		127, 0, BRISTOL_HELLO|synth->voices);
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
 		cleanupBristol();
+
 	/*
 	 * The returned value is the connection ID that we need to use for all
 	 * further system conversations.
 	 */
 	sid = (msg.params.bristol.valueMSB << 7) + msg.params.bristol.valueLSB;
 
+printf("	sid is %i\n", sid);
 	bristolMidiSendMsg(global->controlfd, sid,
 		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
