@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -135,9 +135,14 @@ static int dc, mbh = 0;
  *	RM damaged FIXED
  *	Memories using S&H do not get LFO Mod loaded correctly. FIXED
  *
- *	Key assign modes finished, need more TESTing
- *		Need to clear out keys when changing modes in engine. TEST
- *	Arpeggio finished, needs more testing, esp trig TEST
+ *	Key assign modes finished. FIXED
+ *		Need to clear out keys when changing modes in engine. FIXED
+ *	Arpeggio finished, needs more testing, esp trig FIXED
+ *
+ * Implemented two new arpeggiation modes, Arpeg-1 and Arpeg-2. Both will split
+ * the keyboard at note 52 then Arpeg-1 will apply on the top half, the bottom
+ * half can play duophonic baselines. Arpeg-2 will apply to the bottom half
+ * and the upper half can play duophonic leads.
  *
  * Review volume levels direct/filter and Multi.
  *
@@ -421,12 +426,9 @@ brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/sidbon.xpm", 0},
 	{"", 0,  S2C23+8, S2R0 - 50, VBW3, VBH2, 0, 15, 0,
 		"bitmaps/knobs/knobyellow.xpm", 0, 0},
-	/* Dummies */
-	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
-		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
 	/*
-	 * MODS: 70
+	 * MODS: 69 (was 70)
 	 *
 	 * Env will have adsr plus routing
 	 * LFO will have waveform, rate (0.1-10Hz) and routing
@@ -436,13 +438,15 @@ brightonLocations locations[DEVICE_COUNT] = {
 	 *	wah
 	 */
 	/* Key Mode */
-	{"", 2, S2C0, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"", 2, S2C0-3, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"", 2, S2C0-3, S2R0+60, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"", 2, S2C0-3, S2R0+120, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"", 2, S2C0-3, S2R0+180, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+		"bitmaps/buttons/sidbon.xpm", 0},
+	{"", 2, S2C0-3, S2R0+240, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
 	/* Mods */
@@ -523,8 +527,15 @@ brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/sidbon.xpm", 0},
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
+
+	/*
+	 * This will shadow the version of the memory. Existing memories will have
+	 * a zero in here (0.40) ones saved by this release (0.40.2) will have a
+	 * 1 in there, this allows the GUI to adjust the keymode parameter.
+	 */
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
+
 	/* 110 - These shadow the mod panel */
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
@@ -537,7 +548,7 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
-	{"", 0, S2C14, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 0, 0},
+	{"", 0, S2C14, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm",0,0},
 	{"", 2, S2C13, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 	{"", 2, S2C15, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
@@ -703,14 +714,32 @@ loadMemoryShim(guiSynth *synth)
 				synth->mem.param[78] != 0? 2:
 						synth->mem.param[79] != 0? 3: 0;
 
-	synth->dispatch[70].other1 =
-		synth->mem.param[70] != 0? 0:
-			synth->mem.param[71] != 0? 1:
-				synth->mem.param[72] != 0? 2:
-					synth->mem.param[73] != 0? 3: 0;
+	/* Key mode changed between releases, this recovers them */
+	if (synth->mem.param[109] == 0) {
+		int on;
+
+		synth->dispatch[69].other1 =
+			synth->mem.param[70] != 0? 0:
+				synth->mem.param[71] != 0? 1:
+					synth->mem.param[72] != 0? 2:
+						synth->mem.param[73] != 0? 3: 0;
+		on = 69 + synth->dispatch[69].other1;
+		event.type = BRISTOL_FLOAT;
+		event.value = 0;
+		brightonParamChange(synth->win, 0,
+			70 + synth->dispatch[69].other1, &event);
+		event.value = 1;
+		brightonParamChange(synth->win, 0, on, &event);
+	} else
+		synth->dispatch[69].other1 =
+			synth->mem.param[69] != 0? 0:
+				synth->mem.param[70] != 0? 1:
+					synth->mem.param[71] != 0? 2:
+						synth->mem.param[72] != 0? 3:
+							synth->mem.param[73] != 0? 4: 0;
 
 	bristolMidiSendMsg(global.controlfd, synth->sid, 126, 5,
-		synth->dispatch[70].other1);
+		synth->dispatch[69].other1);
 
 	/* Set up the trigger options */
 
@@ -738,16 +767,16 @@ loadMemoryShim(guiSynth *synth)
 		bristolMidiSendMsg(global.controlfd, synth->sid, 126, 66, 1);
 
 	/* See if we have to force memory variables */
-	if (synth->mem.param[71] != 0)
+	if (synth->mem.param[70] != 0)
 	{
 		int i;
 
 		for (i = 0; i < 20; i++)
 		{
 			event.value = synth->mem.param[i];
+			brightonParamChange(synth->win, 0, i, &event);
 			brightonParamChange(synth->win, 0, i + 20, &event);
-			brightonParamChange(synth->win, 0, i + 20, &event);
-			brightonParamChange(synth->win, 0, i + 20, &event);
+			brightonParamChange(synth->win, 0, i + 40, &event);
 		}
 	}
 }
@@ -810,7 +839,7 @@ sidVoiceShim(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	brightonEvent event;
 	int off1 = 20, off2 = 40;
 
-	if (synth->mem.param[71] != 0)
+	if (synth->mem.param[70] != 0)
 	{
 		if (vexclude)
 			return;
@@ -918,9 +947,9 @@ sidMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	if ((synth->flags & OPERATIONAL) == 0)
 		return;
 
-	if (synth->dispatch[70].other2)
+	if (synth->dispatch[69].other2)
 	{
-		synth->dispatch[70].other2 = 0;
+		synth->dispatch[69].other2 = 0;
 		return;
 	}
 
@@ -932,19 +961,19 @@ sidMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	 * not have much issue with the message, but we are concerned with
 	 * the display.
 	 */
-	if (synth->dispatch[70].other1 != -1)
+	if (synth->dispatch[69].other1 != -1)
 	{
-		synth->dispatch[70].other2 = 1;
+		synth->dispatch[69].other2 = 1;
 
-		if (synth->dispatch[70].other1 != o)
+		if (synth->dispatch[69].other1 != o)
 			event.value = 0;
 		else
 			event.value = 1;
 
 		brightonParamChange(synth->win, synth->panel,
-			synth->dispatch[70].other1 + 70, &event);
+			synth->dispatch[69].other1 + 69, &event);
 	}
-	synth->dispatch[70].other1 = o;
+	synth->dispatch[69].other1 = o;
 
 	bristolMidiSendMsg(fd, synth->sid, 126, 5, o);
 
@@ -954,7 +983,7 @@ sidMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	 * to voice-3 entering Poly-1 and reset them to GUI parameters on exit,
 	 * or do nothing.
 	 */
-	if (synth->mem.param[71] != 0)
+	if (synth->mem.param[70] != 0)
 	{
 		/*
 		 * Sync voices in engine
@@ -1090,6 +1119,7 @@ sidMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->location = 1;
 			if (brightonDoubleClick(dc) != 0)
 			{
+				synth->mem.param[109] = 1.0;
 				saveMemory(synth, SYNTH_NAME, 0,
 					synth->bank * 10 + synth->location, 0);
 				displayText(synth, "PROGRAM",
@@ -1570,12 +1600,13 @@ sidInit(brightonWindow *win)
 	dispatch[68].operator = 3;
 	dispatch[68].routine = (synthRoutine) sidMidiShim;
 
-	dispatch[70].operator = 0;
-	dispatch[71].operator = 1;
-	dispatch[72].operator = 2;
-	dispatch[73].operator = 3;
-	dispatch[70].routine = dispatch[71].routine = dispatch[72].routine =
-		dispatch[73].routine = (synthRoutine) sidMode;
+	dispatch[69].operator = 0;
+	dispatch[70].operator = 1;
+	dispatch[71].operator = 2;
+	dispatch[72].operator = 3;
+	dispatch[73].operator = 4;
+	dispatch[69].routine = dispatch[70].routine = dispatch[71].routine
+		= dispatch[72].routine = dispatch[73].routine = (synthRoutine) sidMode;
 
 	/* Mod freq, gain and waveform */
 	dispatch[74].controller = 126;
@@ -1717,7 +1748,7 @@ sidInit(brightonWindow *win)
 	/* Filter */
 
 	dispatch[MEM_START].other1 = 1;
-	dispatch[70].other1 = 0;
+	dispatch[69].other1 = 0;
 	dispatch[76].other1 = 0;
 
 	return(0);
@@ -1781,7 +1812,7 @@ sidConfigure(brightonWindow *win)
 	/* First memory location */
 	event.value = 1.0;
 	brightonParamChange(synth->win, 0, MEM_START + synth->location-1, &event);
-	brightonParamChange(synth->win, 0, 70, &event);
+	brightonParamChange(synth->win, 0, 69, &event);
 	brightonParamChange(synth->win, 0, 76, &event);
 
 	loadMemory(synth, SYNTH_NAME, 0, synth->bank * 10 + synth->location,
