@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -931,6 +931,11 @@ printf("going operational\n");
 	event.value = 1;
 	brightonParamChange(synth->win, MEM_PANEL, 0, &event);
 
+	event.command = BRIGHTON_PARAMCHANGE;
+	event.type = BRIGHTON_FLOAT;
+	event.value = 0;
+	brightonParamChange(synth->win, KEY_PANEL, 0, &event);
+
 	dc = brightonGetDCTimer(win->dcTimeout);
 
 	return(0);
