@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -687,11 +687,12 @@ voxInit(brightonWindow* win)
 
 	synth->dispatch[OPTS_START + 1].routine
 		= synth->dispatch[OPTS_START + 2].routine
-		= synth->dispatch[OPTS_START + 3].routine
 			= (synthRoutine) voxOption;
 	synth->dispatch[OPTS_START + 1].controller = 0;
 	synth->dispatch[OPTS_START + 2].controller = 1;
-	synth->dispatch[OPTS_START + 3].controller = 2;
+
+	synth->dispatch[OPTS_START + 3].controller = 0;
+	synth->dispatch[OPTS_START + 3].operator = 5;
 
 	synth->dispatch[MOD_START + 0].routine
 		= synth->dispatch[MOD_START + 1].routine
@@ -732,7 +733,7 @@ voxInit(brightonWindow* win)
 	 */
 	bristolMidiSendMsg(global.controlfd, synth->sid, 1, 0, 550);
 	bristolMidiSendMsg(global.controlfd, synth->sid, 1, 1, 1600);
-	bristolMidiSendMsg(global.controlfd, synth->sid, 1, 2, 1);
+	bristolMidiSendMsg(global.controlfd, synth->sid, 1, 2, 0);
 
 	return(0);
 }
