@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -414,7 +414,7 @@ static int operate(bristolOP *operator,
 		 */
 		gdelta = wtp - ((float) ((int) wtp));
 
-		if (wtp == ARPDCO_WAVE_SZE_M) {
+		if (wtp >= ARPDCO_WAVE_SZE_M) {
 			ob[obp] += (wt[(int) wtp] + (wt[0] - wt[(int) wtp])
 				* gdelta) * gain;
 			obsin[obp] += (sinwave[(int) wtp]
@@ -490,7 +490,7 @@ static int operate(bristolOP *operator,
 		 */
 		if ((wtp += ib[obp] * transp) >= ARPDCO_WAVE_SZE)
 		{
-			while (wtp > ARPDCO_WAVE_SZE)
+			while (wtp >= ARPDCO_WAVE_SZE)
 				wtp -= ARPDCO_WAVE_SZE;
 		}
 
