@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -39,6 +39,48 @@ static void a440();
  */
 int s440holder = 0;
 
+static void
+bristolUnlinkVoice(audioMain *audiomain, bristolVoice *v)
+{
+	bristolVoice *vtp;
+
+	/*
+	 * See if this is a monophonic voice with note preference
+	 */
+	if ((v->baudio != NULL) && (v->baudio->voicecount == 1)
+		&& (v->baudio->notemap.flags != 0))
+	{
+		v->flags &= ~BRISTOL_KEYDONE;
+		v = v->next;
+		return;
+	}
+
+	/*
+	 * Move to the tail of the freelist, unlink it first:
+	 */
+	if (v->next != NULL)
+		v->next->last = v->last;
+	else
+		audiomain->playlast = v->last;
+
+	if (v->last != NULL)
+		v->last->next = v->next;
+	else
+		audiomain->playlist = v->next;
+
+	vtp = v;
+	/* We have to move this to the next entry now */
+	v = v->next;
+
+	/* Unlinked, put it on the tail of the freelist */
+	vtp->next = NULL;
+	if ((vtp->last = audiomain->freelast) != NULL)
+		vtp->last->next = vtp;
+	else
+		audiomain->freelist = vtp;
+	audiomain->freelast = vtp;
+}
+
 /*
  * This should be organised to be a callback for the Jack and DSSI interfaces.
  * there may be issues of internal buffering that will have to be reviewed, and
@@ -58,7 +100,7 @@ int s440holder = 0;
 int
 doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 {
-	register bristolVoice *voice = audiomain->playlist;
+	register bristolVoice *voice;
 	register int i;
 	register Baudio *thisaudio;
 	register float *extmult, *leftch, *rightch, gain;
@@ -77,7 +119,9 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	 * Configure flags to force one iteration of the preops, and to enable
 	 * postops if a voice is active.
 	 */
-	thisaudio = audiomain->audiolist;
+	if ((thisaudio = audiomain->audiolist) == NULL)
+		return(0);
+
 	while (thisaudio != NULL)
 	{
 		if (bristolActiveSenseCheck(audiomain, thisaudio) == 0)
@@ -95,13 +139,10 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 
 			freeBristolAudio(audiomain, holder);
 
-			if (audiomain->audiolist == NULL)
+			if ((audiomain->audiolist == NULL)
+				&& (audiomain->flags & BRISTOL_MIDI_WAIT))
 			{
-				/*
-				 * Last synth GUI has exit, we may as well do the same.
-				 */
-				printf("empty audio list, exiting\n");
-				inthandler();
+				audiomain->atStatus = BRISTOL_TERM;
 				return(0);
 			}
 
@@ -109,7 +150,7 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			 * Otherwise we need to make sure none of the voices are using this
 			 * removed audio.
 			 */
-			for (vl = voice; vl != NULL; vl = vl->next)
+			for (vl = audiomain->playlist; vl != NULL; vl = vl->next)
 				if (vl->baudio == holder)
 					vl->baudio = NULL;
 
@@ -129,29 +170,27 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		thisaudio = thisaudio->next;
 	}
 
-	if (audiomain->audiolist != NULL)
-	{
-		/*
-		 * a440 has to be done here, since it must be active even when no keys
-		 * are pressed.
-		 */
-		if (audiomain->audiolist->mixflags & A_440)
-			a440(audiomain, outbuf, audiomain->samplecount);
-	}
+	/*
+	 * a440 has to be done here, since it must be active even when no keys
+	 * are pressed.
+	 */
+	if ((audiomain->audiolist != NULL)
+			&& (audiomain->audiolist->mixflags & A_440))
+		a440(audiomain, outbuf, audiomain->samplecount);
 
 	/*
 	 * Get the long semaphore over the whole polyphonic process
 	 */
-	sem_wait(&audiomain->sem_long);
+	sem_wait(audiomain->sem_long);
 
 	if (audiomain->newlist != NULL)
 	{
-		bristolVoice *v = voice, *vtp;
+		bristolVoice *v = audiomain->playlist, *vtp;
 
 		/*
 		 * Get the short semaphore over the new voice management sequences
 		 */
-		sem_wait(&audiomain->sem_short);
+		sem_wait(audiomain->sem_short);
 
 		/*
 		 * Move the new voices to the playlist. Then do a scan of the playlist
@@ -166,8 +205,8 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 				/*
 				 * See if this is a monophonic voice with note preference
 				 */
-				if ((voice->baudio != NULL) && (voice->baudio->voicecount == 1)
-					&& (voice->baudio->notemap.flags != 0))
+				if ((v->baudio != NULL) && (v->baudio->voicecount == 1)
+					&& (v->baudio->notemap.flags != 0))
 				{
 					v->flags &= ~BRISTOL_KEYDONE;
 					v = v->next;
@@ -230,11 +269,11 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		/*
 		 * Free the short semaphore over the new voice management sequences
 		 */
-		sem_post(&audiomain->sem_short);
-
-		voice = audiomain->playlist;
+		sem_post(audiomain->sem_short);
 	}
 
+	voice = audiomain->playlist;
+
 	/*
 	 * We need to look through our voice list, and see if any are active. If
 	 * so then start running the voice structures through the sound structures.
@@ -245,6 +284,10 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	{
 		if (voice->baudio == NULL)
 		{
+			/*
+			 * Unlink this voice
+			 */
+			bristolUnlinkVoice(audiomain, voice);
 			voice = voice->next;
 			continue;
 		}
@@ -292,7 +335,8 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 				 * This looks a bit brutal however the assignement code should
 				 * avoid this situation.
 				 */
-printf("overvoice keydone\n");
+				if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+					printf("overvoice keydone\n");
 				voice->flags |= BRISTOL_KEYDONE;
 				voice = voice->next;
 				continue;
@@ -302,10 +346,11 @@ printf("overvoice keydone\n");
 			 * If we are arpeggiating or sequencing we need to check on rolling
 			 * this voice along
 			 */
-			if ((voice->baudio->mixflags
+			if (((voice->baudio->mixflags
 				& (BRISTOL_SEQUENCE|BRISTOL_ARPEGGIATE))
 				&& (bristolArpegReVoice(voice->baudio, voice,
 					(float) audiomain->samplerate) < 0))
+				|| ((voice->locals == 0) || (voice->locals[voice->index] == 0)))
 			{
 				voice = voice->next;
 				continue;
@@ -328,11 +373,6 @@ printf("overvoice keydone\n");
 		voice = voice->next;
 	}
 
-	/*
-	 * Free the long semaphore over the polyphonic process.
-	 */
-	sem_post(&audiomain->sem_long);
-
 	/*
 	 * See if any of the voices have postoperators configured.
 	 */
@@ -355,6 +395,11 @@ printf("overvoice keydone\n");
 		voice = voice->next;
 	}
 
+	/*
+	 * Free the long semaphore over the polyphonic process.
+	 */
+	sem_post(audiomain->sem_long);
+
 	/*
 	 * At this point all the voices have put their output onto the leftbuf and
 	 * rightbuf (leftbuf only if mono). We should go through the baudio 
