@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -131,6 +131,7 @@ audioThread(audioMain *audiomain)
 		_exit(0);
 #endif /* _BRISTOL_JACK */
 	}
+
 	/*
 	 * open the audio device with our desired buffer size, and take its
 	 * returned fragment size.
@@ -216,7 +217,7 @@ audioThread(audioMain *audiomain)
 		}
 
 		/*
-		 * The startbuf is actually interleaved at the moment, something whch
+		 * The startbuf is actually interleaved at the moment, something which
 		 * needs to be changed.
 		 */
 		if (audiomain->ingain > 1)
@@ -364,15 +365,12 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 				baudio->notemap.flags |= BRISTOL_MNL_LNP;
 			else if ((int) (value * CONTROLLER_RANGE) == 2)
 				baudio->notemap.flags |= BRISTOL_MNL_HNP;
-#warning the two printf below should be taken out completely
-//printf("MNL 1 flags %x %f %f\n", baudio->notemap.flags, (int) (value * CONTROLLER_RANGE), value);
 			return(0);
 		case BRISTOL_NRP_MNL_TRIG:
 			if (value != 0)
 				baudio->notemap.flags |= BRISTOL_MNL_TRIG;
 			else
 				baudio->notemap.flags &= ~BRISTOL_MNL_TRIG;
-//printf("MNL 2 flags %x\n", baudio->notemap.flags);
 			return(0);
 		case BRISTOL_NRP_MNL_VELOC:
 			if (value != 0)
@@ -387,26 +385,30 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 			 * for the emulator. This does not have to be an issue, it allows
 			 * us to view what each emulation is delivered by midi through the
 			 * bristol API.
+			 *
+			 * 052009 - integrate code to request the MIDI library include byte
+			 * debuging of interfaces.
 			 */
 			if (value == 0)
 				baudio->midiflags &= ~(BRISTOL_MIDI_DEBUG1|BRISTOL_MIDI_DEBUG2);
-			else if (((int) (value * CONTROLLER_RANGE)) == 1) {
+			else if (((int) (value * C_RANGE_MIN_1)) == 1) {
 				baudio->midiflags |= BRISTOL_MIDI_DEBUG1;
 				baudio->midiflags &= ~BRISTOL_MIDI_DEBUG2;
 			} else
 				baudio->midiflags |= (BRISTOL_MIDI_DEBUG2|BRISTOL_MIDI_DEBUG1);
+
+			bristolMidiOption(0, BRISTOL_NRP_DEBUG,
+				(value * C_RANGE_MIN_1) > 1? (int) 1: (int) 0);
+
 			return(0);
 		case BRISTOL_NRP_ENABLE_NRP:
 			if (value != 0)
 				baudio->midiflags |= BRISTOL_MIDI_NRP_ENABLE;
 			else
 				baudio->midiflags &= ~BRISTOL_MIDI_NRP_ENABLE;
-/*			printf("NRP flags %x from %f\n", baudio->midiflags, value); */
 			return(0);
 	}
 
-/* printf("detune %f, gain %f, pitch %i\n", baudio->detune, baudio->gain, baudio->midi_pitch); */
-
 	/*
 	 * This should also consider indexing this NRP value into an arbitrary
 	 * engine control? This is non-trivial since there is not a table to 
