@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -463,7 +463,7 @@ static int operate(bristolOP *operator,
 		 * Take a sample from the wavetable into the output buffer, this is
 		 * the combined tri/ramp waveform.
 		 */
-		if (((int) wtp) == BITONE_WAVE_SZE_M) {
+		if (((int) wtp) >= BITONE_WAVE_SZE_M) {
 			ob[obp] += (wt1[0] * (wtp - ((float) ((int) wtp)))
 				+ wt1[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))));
 			sob[obp] = (wt2[0] * (wtp - ((float) ((int) wtp)))
