@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -201,7 +201,7 @@ baudio->effect[0]->param->param[5].float_val);
 	return(0);
 }
 
-extern void thesermon(int, float *, int);
+extern void thesermon(int, int);
 extern void therequiem(float *, float *, int);
 /*
  * Dosermon is a cheat, sort of. If we are running multiple synths, all using
@@ -219,7 +219,7 @@ bristolVoice *voice, register float *startbuf)
 /*	samplerate = audiomain->samplerate; */
 
 	if ((baudio->mixflags & HAMMOND_SERMON) && dosermon)
-		thesermon(audiomain->samplecount, &baudio->ctab[0], sineform);
+		thesermon(audiomain->samplecount, sineform);
 
 	dosermon = 0;
 
@@ -291,17 +291,7 @@ bristolVoice *voice, register float *startbuf)
 	 */
 
 	flags = voice->flags;
-/*
-	if (baudio->lvoices != 0)
-	{
-		if (flags & BRISTOL_KEYON)
-			voice->flags &= ~BRISTOL_KEYON;
-		if (flags & BRISTOL_KEYREON)
-			voice->flags &= ~BRISTOL_KEYREON;
-		if (flags & BRISTOL_KEYOFF)
-			voice->flags &= ~BRISTOL_KEYOFF;
-	}
-*/
+
 	if (flags & (BRISTOL_KEYON|BRISTOL_KEYREON))
 	{
 		if (baudio->lvoices != 0)
@@ -559,8 +549,9 @@ bristolVoice *voice, register float *startbuf)
 int
 destroyOneHammondVoice(audioMain *audiomain, Baudio *baudio)
 {
-	bristolfree(tmpbuf1);
-	bristolfree(tmpbuf2);
+printf("removing hammond sound\n");
+//	bristolfree(tmpbuf1);
+//	bristolfree(tmpbuf2);
 
 	baudio->mixlocals = NULL;
 	return(0);
@@ -569,11 +560,12 @@ destroyOneHammondVoice(audioMain *audiomain, Baudio *baudio)
 int
 destroyOneHammondB3Voice(audioMain *audiomain, Baudio *baudio)
 {
-	bristolfree(tmpbuf3);
-	bristolfree(tmpbuf4);
+printf("removing B3 sound\n");
+//	bristolfree(tmpbuf3);
+//	bristolfree(tmpbuf4);
 
-	tmpbuf3 = NULL;
-	tmpbuf4 = NULL;
+//	tmpbuf3 = NULL;
+//	tmpbuf4 = NULL;
 
 	baudio->mixlocals = NULL;
 	return(0);
