@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -158,6 +158,11 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 	if (operator != 126)
 		return(0);
 
+	if (baudio->detune != smods->detune) {
+		sid_IO(smods->sidid[AUD_SID], B_SID_DETUNE, baudio->detune * 1000);
+		smods->detune = baudio->detune;
+	}
+
 	switch (controller) {
 		case 0:
 			baudio->glide = value * value * baudio->glidemax;
@@ -187,15 +192,14 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 			smods->keymode = ivalue;
 			sidClearVoices(smods);
 			break;
-		/* Arpeggiator speed and modes */
 		case 6:
 			/*
+			 * Arpeggiator speed and modes:
 			 * Want to go from something ridiculously small to max 250ms, this
 			 * is the number of samples between frequency steps, triggers, 
-			 * waveform scan, etc. The 16 is fixed for now, it is the optimised
-			 * operate() cycle count.
+			 * waveform scan, etc.
 			 */
-			smods->arpegspeed = 16 + value * value * baudio->samplerate / 4;
+			smods->arpegspeed = 800 + value * value * baudio->samplerate / 4;
 			break;
 		case 7:
 			/* Retrig */
@@ -242,8 +246,7 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 			 * not just a couple of octaves.
 			 */
 			smods->tune[B_SID_VOICE_1] = 1.0
-				+ (baudio->note_diff - 1)
-				* (value * 2 - 1);
+				+ (baudio->note_diff - 1) * (value * 2 - 1);
 			if (smods->ckey[B_SID_VOICE_1] != 0)
 				sidAssignVoice(smods, B_SID_VOICE_1, 0);
 			break;
@@ -259,7 +262,7 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 		case 17:
 			/* Glide - emulator parameter */
 			smods->glide[B_SID_VOICE_1] =
-				value * value * 5 * baudio->samplerate / 16;
+				value * value * value * 5 * baudio->samplerate / 16;
 			break;
 		case 18:
 			sidflag(smods, AUD_SID, B_SID_V1_CONTROL, B_SID_V_RINGMOD, ivalue);
@@ -327,7 +330,7 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 		case 37:
 			/* Glide - emulator parameter */
 			smods->glide[B_SID_VOICE_2] =
-				value * value * 5 * baudio->samplerate / 16;
+				value * value * value * 5 * baudio->samplerate / 16;
 			break;
 		case 38:
 			sidflag(smods, AUD_SID, B_SID_V2_CONTROL, B_SID_V_RINGMOD, ivalue);
@@ -394,7 +397,7 @@ sidController(Baudio *baudio, u_char operator, u_char controller, float value)
 		case 57:
 			/* Glide - emulator parameter */
 			smods->glide[B_SID_VOICE_3] =
-				value * value * 5 * baudio->samplerate / 16;
+				value * value * value * 5 * baudio->samplerate / 16;
 			break;
 		case 58:
 			sidflag(smods, AUD_SID, B_SID_V3_CONTROL, B_SID_V_RINGMOD, ivalue);
@@ -729,12 +732,15 @@ sidAssignVoice(sidmods *smods, int voice, int flags)
 	while (modf > 65535)
 		modf = modf >> 1;
 
+	//printf("%i -> %i from %f %f (%i) [%i]\n", nf, modf, smods->tune[voice],
+	//smods->transpose[voice], smods->ckey[voice], freqmap[smods->ckey[voice]]);
+
 	if ((smods->glide[voice] == 0) || (flags & B_SID_RENEW))
 	{
 		sid_register(smods->sidid[AUD_SID], offset+B_SID_V1_FREQ_LO,
 			modf & 0x0ff);
 		sid_register(smods->sidid[AUD_SID], offset+B_SID_V1_FREQ_HI,
-		modf >> 8);
+			modf >> 8);
 
 		smods->sidreg[AUD_SID][offset + B_SID_V1_FREQ_LO] = modf & 0x0ff;
 		smods->sidreg[AUD_SID][offset + B_SID_V1_FREQ_HI] = (modf >> 8) & 0x0ff;
@@ -839,12 +845,6 @@ sidPoly1NoteLogic(Baudio *baudio, sidmods *smods)
 					sidAssignVoice(smods, B_SID_VOICE_3, B_SID_TRIG);
 				}
 
-				printf("nk %i %i: %i %i %i: %i %i %i\n", smods->ccount, i,
-					low, middle, high,
-					smods->ckey[B_SID_VOICE_1],
-					smods->ckey[B_SID_VOICE_2],
-					smods->ckey[B_SID_VOICE_3]);
-
 				smods->ccount++;
 			} else {
 				/*
@@ -854,12 +854,6 @@ sidPoly1NoteLogic(Baudio *baudio, sidmods *smods)
 				smods->lvel = smods->velocity[middle]
 					= baudio->notemap.velocity[i] / 127.0;
 				sidAssignVoice(smods, middle, B_SID_TRIG);
-
-				printf("rk %i %i: %i %i %i: %i %i %i\n", smods->ccount, i,
-					low, middle, high,
-					smods->ckey[B_SID_VOICE_1],
-					smods->ckey[B_SID_VOICE_2],
-					smods->ckey[B_SID_VOICE_3]);
 			}
 
 			smods->localmap[0][i] = i;
@@ -887,12 +881,6 @@ sidPoly1NoteLogic(Baudio *baudio, sidmods *smods)
 
 			smods->localmap[0][i] = 0;
 
-			printf("ck %i %i: %i %i %i: %i %i %i\n", smods->ccount, i,
-				low, middle, high,
-				smods->ckey[B_SID_VOICE_1],
-				smods->ckey[B_SID_VOICE_2],
-				smods->ckey[B_SID_VOICE_3]);
-
 			if (smods->ccount < 0)
 				smods->ccount = 0;
 
@@ -913,7 +901,7 @@ sidPoly1NoteLogic(Baudio *baudio, sidmods *smods)
 						&& (smods->ckey[B_SID_VOICE_2] != j)
 						&& (smods->ckey[B_SID_VOICE_3] != j))
 					{
-						printf("Reapply note %i\n", j);
+						//printf("Reapply note %i\n", j);
 						smods->ckey[fv] = j;
 						smods->velocity[middle]
 							= baudio->notemap.velocity[j] / 127.0;
@@ -928,17 +916,15 @@ sidPoly1NoteLogic(Baudio *baudio, sidmods *smods)
 }
 
 /*
- * If we have less than 3 voices assigned just look for the next free voice.
- * If not look for the middle note.
- *
- *	flag voice as occupying this midi note, 
- *	flag the midi note as known,
+ * Assign highest two notes to whatever voices are avilable, the lowest three
+ * notes (that do not conflict) to the arpeggiator. Only arpeggiate if there
+ * are three notes available.
  */
 static void
-sidPoly3NoteLogic(Baudio *baudio, sidmods *smods)
+sidPoly_3NoteLogic(Baudio *baudio, sidmods *smods)
 {
 	int i, ac = 0;
-	int occ = 0;
+	//int occ = 0;
 
 	bzero(smods->arpegtable, B_SID_ARPEG_MAX * sizeof(unsigned short));
 
@@ -1029,6 +1015,7 @@ sidPoly3NoteLogic(Baudio *baudio, sidmods *smods)
 		}
 	}
 
+/*
 	if (occ)
 	{
 		printf("low %i, high %i\narpeg:",
@@ -1037,6 +1024,268 @@ sidPoly3NoteLogic(Baudio *baudio, sidmods *smods)
 			printf(" %i,", smods->arpegtable[ac]);
 		printf("\n");
 	}
+*/
+}
+
+/*
+ * Take a fixed split point, below this point stuff the arpeggiation table, 
+ * above this point use duophonic note preference - we should consider always
+ * having the voices active if there is a note pressed, acting as a monophonic
+ * with two voices where there is only one note.
+ */
+static void
+sidPoly3NoteLogic(Baudio *baudio, sidmods *smods)
+{
+	int i, ac = 0;
+	//int occ = 0;
+
+	bzero(smods->arpegtable, B_SID_ARPEG_MAX * sizeof(unsigned short));
+
+	/* 52 is middle-e on the GUI */
+	for (i = 127; i >= 52; i--)
+	{
+		if (baudio->notemap.key[i] != 0)
+			smods->arpegtable[ac++] = i;
+		smods->localmap[0][i] = baudio->notemap.key[i] == 0? 0:i;
+	}
+
+	ac = 0;
+
+	for (i = 1; i < 52; i++)
+	{
+		/*
+		 * There are 3 cases:
+		 *
+		 * 	a. We do not know this key:
+		 * 		Assign to a voice
+		 * 	b. We do know this key, its assigned - do nothing.
+		 * 	c. We do know this key, its gone:
+		 * 		Free voice if it was Voice-1 or Voice-3
+		 */
+		if (baudio->notemap.key[i] > 0)
+		{
+			if (smods->localmap[0][i] != i)
+			{
+				int voice = B_SID_VOICE_3;
+
+				/*
+				 * Not a known note - new , Check V1 and V3
+				 *	if V1 == 0, take it
+				 *	if not is V3 == 0, take it
+				 *	if not take highest of V1, V3
+				 */
+				if ((smods->ckey[B_SID_VOICE_1] == 0) ||
+					((smods->ckey[B_SID_VOICE_3] != 0) &&
+					(smods->ckey[B_SID_VOICE_1] > smods->ckey[B_SID_VOICE_3])))
+					voice = B_SID_VOICE_1;
+
+				smods->ckey[voice] = i;
+				smods->lvel = smods->velocity[voice]
+					= baudio->notemap.velocity[i] / 127.0;
+				sidAssignVoice(smods, voice, B_SID_TRIG);
+				smods->ccount++;
+				smods->localmap[0][i] = i;
+			}
+
+			smods->localmap[0][i] = i;
+		} else
+			if ((baudio->notemap.key[i] <= 0) && (smods->localmap[0][i] == i))
+		{
+			/* Was a known key? note off only if allocated */
+			if (smods->ckey[B_SID_VOICE_1] == i)
+			{
+				sidAssignVoice(smods, B_SID_VOICE_1, B_SID_REL);
+				smods->ckey[B_SID_VOICE_1] = 0;
+				smods->ccount--;
+				//occ++;
+			} else if (smods->ckey[B_SID_VOICE_3] == i) {
+				sidAssignVoice(smods, B_SID_VOICE_3, B_SID_REL);
+				smods->ckey[B_SID_VOICE_3] = 0;
+				smods->ccount--;
+				//occ++;
+			}
+
+			smods->localmap[0][i] = 0;
+
+			if (smods->ccount < 0)
+				smods->ccount = 0;
+		}
+	}
+
+/*
+	if (occ)
+	{
+		printf("low %i, high %i\narpeg:",
+			smods->ckey[B_SID_VOICE_1], smods->ckey[B_SID_VOICE_3]);
+		for (ac = 0; smods->arpegtable[ac] != 0; ac++)
+			printf(" %i,", smods->arpegtable[ac]);
+		printf("\n");
+	}
+*/
+}
+
+/*
+ * Take a fixed split point, below this point stuff the arpeggiation table, 
+ * above this point use duophonic note preference - we should consider always
+ * having the voices active if there is a note pressed, acting as a monophonic
+ * with two voices where there is only one note.
+ */
+static void
+sidPoly4NoteLogic(Baudio *baudio, sidmods *smods)
+{
+	int i, ac = 0;
+	//int occ = 0;
+
+	bzero(smods->arpegtable, B_SID_ARPEG_MAX * sizeof(unsigned short));
+
+	/* 52 is middle-e on the GUI */
+	for (i = 1; i < 52; i++)
+	{
+		if (baudio->notemap.key[i] != 0)
+			smods->arpegtable[ac++] = i;
+		smods->localmap[0][i] = baudio->notemap.key[i] == 0? 0:i;
+	}
+
+	ac = 0;
+
+	for (i = 127; i >= 52; i--)
+	{
+		/*
+		 * There are 3 cases:
+		 *
+		 * 	a. We do not know this key:
+		 * 		Assign to a voice
+		 * 	b. We do know this key, its assigned - do nothing.
+		 * 	c. We do know this key, its gone:
+		 * 		Free voice if it was Voice-1 or Voice-3
+		 */
+		if (baudio->notemap.key[i] > 0)
+		{
+			if (smods->localmap[0][i] != i)
+			{
+				int voice = B_SID_VOICE_3;
+
+				/*
+				 * Not a known note - new , Check V1 and V3
+				 *	if V1 == 0, take it
+				 *	if not is V3 == 0, take it
+				 *	if not take lowest of V1, V3
+				 */
+				if ((smods->ckey[B_SID_VOICE_1] == 0) ||
+					((smods->ckey[B_SID_VOICE_3] != 0) && 
+					(smods->ckey[B_SID_VOICE_1] < smods->ckey[B_SID_VOICE_3])))
+					voice = B_SID_VOICE_1;
+
+				smods->ckey[voice] = i;
+				smods->lvel = smods->velocity[voice]
+					= baudio->notemap.velocity[i] / 127.0;
+				sidAssignVoice(smods, voice, B_SID_TRIG);
+				smods->ccount++;
+				smods->localmap[0][i] = i;
+			}
+
+			smods->localmap[0][i] = i;
+		} else if ((baudio->notemap.key[i] <= 0)
+			&& (smods->localmap[0][i] == i))
+		{
+			/* Was a known key? note off if allocated */
+			if (smods->ckey[B_SID_VOICE_1] == i)
+			{
+				sidAssignVoice(smods, B_SID_VOICE_1, B_SID_REL);
+				smods->ckey[B_SID_VOICE_1] = 0;
+				smods->ccount--;
+				//occ++;
+			} else if (smods->ckey[B_SID_VOICE_3] == i) {
+				sidAssignVoice(smods, B_SID_VOICE_3, B_SID_REL);
+				smods->ckey[B_SID_VOICE_3] = 0;
+				smods->ccount--;
+				//occ++;
+			}
+
+			smods->localmap[0][i] = 0;
+
+			if (smods->ccount < 0)
+				smods->ccount = 0;
+		}
+	}
+
+/*
+	if (occ)
+	{
+		printf("low %i, high %i\narpeg:",
+			smods->ckey[B_SID_VOICE_1], smods->ckey[B_SID_VOICE_3]);
+		for (ac = 0; smods->arpegtable[ac] != 0; ac++)
+			printf(" %i,", smods->arpegtable[ac]);
+		printf("\n");
+	}
+*/
+}
+
+static int
+sidCheckModTrigger(sidmods *smods)
+{
+	if ((smods->flags & (B_SID_E_GATE_V1|B_SID_E_GATE_V2|B_SID_E_GATE_V3)) == 0)
+		return(0);
+
+	switch (smods->keymode) {
+		default:
+		case B_S_KEYMODE_MONO:
+		case B_S_KEYMODE_POLY1:
+			/* All notes are the same - trigger */
+			if (smods->flags
+				& (B_SID_E_GATE_V1|B_SID_E_GATE_V2|B_SID_E_GATE_V3))
+				return(1);
+			break;
+		case B_S_KEYMODE_POLY2:
+			/*
+			 * Check if Envmod affects this voice - mod routing, all voices
+			 *
+			 *	Do we go to filter and mod goes to filter? Trig
+			 *	Does mod go to this voice? Trig
+			 */
+			if ((smods->flags & B_SID_E_GATE_V1) &&
+				/* Filter match */
+				(((smods->sidreg[AUD_SID][B_SID_FILT_RES_F] & B_SID_F_MIX_V_1)
+					&& (smods->modrouting & S_MOD_ENV_FILTER))
+				|| /* Mod me match */
+					(smods->modrouting & (S_MOD_ENV_V1_PW|S_MOD_ENV_V1_FREQ))))
+				return(1);
+			if ((smods->flags & B_SID_E_GATE_V2) &&
+				/* Filter match */
+				(((smods->sidreg[AUD_SID][B_SID_FILT_RES_F] & B_SID_F_MIX_V_2)
+					&& (smods->modrouting & S_MOD_ENV_FILTER))
+				|| /* Mod me match */
+					(smods->modrouting & (S_MOD_ENV_V2_PW|S_MOD_ENV_V2_FREQ))))
+				return(1);
+			if ((smods->flags & B_SID_E_GATE_V3) &&
+				/* Filter match */
+				(((smods->sidreg[AUD_SID][B_SID_FILT_RES_F] & B_SID_F_MIX_V_3)
+					&& (smods->modrouting & S_MOD_ENV_FILTER))
+				|| /* Mod me match */
+					(smods->modrouting & (S_MOD_ENV_V3_PW|S_MOD_ENV_V3_FREQ))))
+				return(1);
+			break;
+		case B_S_KEYMODE_POLY3: /* Arpeg-1 */
+		case B_S_KEYMODE_POLY4: /* Arpeg-2 */
+			/* Check if Envmod affects this voice - mod routing, voice-1/3 */
+			if ((smods->flags & B_SID_E_GATE_V1) &&
+				/* Filter match */
+				(((smods->sidreg[AUD_SID][B_SID_FILT_RES_F] & B_SID_F_MIX_V_1)
+					&& (smods->modrouting & S_MOD_ENV_FILTER))
+				|| /* Mod me match */
+					(smods->modrouting & (S_MOD_ENV_V1_PW|S_MOD_ENV_V1_FREQ))))
+				return(1);
+			if ((smods->flags & B_SID_E_GATE_V3) &&
+				/* Filter match */
+				(((smods->sidreg[AUD_SID][B_SID_FILT_RES_F] & B_SID_F_MIX_V_3)
+					&& (smods->modrouting & S_MOD_ENV_FILTER))
+				|| /* Mod me match */
+					(smods->modrouting & (S_MOD_ENV_V3_PW|S_MOD_ENV_V3_FREQ))))
+				return(1);
+			break;
+	}
+
+	return(0);
 }
 
 /*
@@ -1115,17 +1364,15 @@ bristolVoice *voice, register float *startbuf)
 					= baudio->notemap.velocity[voice->key.key] / 127.0;
 		}
 	} else if ((smods->keymode == B_S_KEYMODE_POLY1)
-		|| (smods->keymode == B_S_KEYMODE_POLY2)) {
+		|| (smods->keymode == B_S_KEYMODE_POLY2))
 		/* Polyphonic voice allocation, no arpeggiation */
 		sidPoly1NoteLogic(baudio, smods);
-	} else {
-		/*
-		 * Poly3 has yet to be done, it should find the highest and lowest keys
-		 * and assign them to voice-1 and voice-3 respectively, then build an
-		 * arpeggiator map for voice-2 to scan
-		 */
+	else if (smods->keymode == B_S_KEYMODE_POLY3)
+		/* High split */
 		sidPoly3NoteLogic(baudio, smods);
-	}
+	else
+		/* Low Split */
+		sidPoly4NoteLogic(baudio, smods);
 
 	if (smods->modrouting & S_MOD_T_LFO_RATE)
 	{
@@ -1150,13 +1397,10 @@ bristolVoice *voice, register float *startbuf)
 	for (i = 0; i < audiomain->samplecount; i+=16)
 	{
 		/* 
-		 * We retrigger the MOD env only on specific keys
+		 * We retrigger the MOD env only on specific keys: Note that we do not
+		 * trigger the MOD env on arpeggiation steps.
 		 */
-		if (((smods->keymode >= B_S_KEYMODE_POLY3)
-				&& (smods->flags & (B_SID_E_GATE_V1|B_SID_E_GATE_V3)))
-			|| ((smods->keymode < B_S_KEYMODE_POLY3)
-				&& (smods->flags
-					& (B_SID_E_GATE_V1|B_SID_E_GATE_V2|B_SID_E_GATE_V3))))
+		if (sidCheckModTrigger(smods))
 		{
 			sidflag(smods, MOD_SID, B_SID_V3_CONTROL, B_SID_V_GATE, 0);
 			smods->modout =
@@ -1782,15 +2026,17 @@ bristolSidInit(audioMain *audiomain, Baudio *baudio)
 	 * when it is done there is still no harm in setting these up here anyway.
 	 */
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
-		B_SID_SN_LEAKAGE, 0.3);
+		B_SID_SN_LEAKAGE, 0.1);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
-		B_SID_SN_RATIO, 0.2);
+		B_SID_SN_RATIO, 0.06);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
-		B_SID_DETUNE, 10); /* Cents */
+		B_SID_DETUNE, 40); /* Cents */
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
 		B_SID_DC_BIAS, 0.3);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
 		B_SID_OBERHEIM, 0.3);
+	sid_IO(((sidmods *) baudio->mixlocals)->sidid[AUD_SID],
+		B_SID_DETUNE, baudio->detune * 1000);
 
 	/* The MOD_SID is going to be noisy */
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[MOD_SID],
@@ -1798,7 +2044,7 @@ bristolSidInit(audioMain *audiomain, Baudio *baudio)
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[MOD_SID],
 		B_SID_SN_RATIO, 0.9);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[MOD_SID],
-		B_SID_DETUNE, 10);
+		B_SID_DETUNE, 0);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[MOD_SID],
 		B_SID_DC_BIAS, 0.5);
 	sid_IO(((sidmods *) baudio->mixlocals)->sidid[MOD_SID],
