@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -24,22 +24,25 @@
 
 #include <bristol.h>
 #include <bristolmidiapi.h>
+#include <bristolmidiapidata.h>
 #include <bristolmidi.h>
 
+extern void inthandler();
 extern void initBristolAudio();
+extern int bristolJackInterface();
 extern void allNotesOff();
 
 int
-getsid(Baudio *baudio, int sid)
+bristolgetsid(Baudio *baudio, int sid)
 {
 	int tsid;
 
 	if (baudio)
-		tsid = getsid(baudio->next, sid);
+		tsid = bristolgetsid(baudio->next, sid);
 	else
 		return(sid);
 
-/*	printf("getsid(%x, %i): %i\n", baudio, sid, tsid); */
+/*	printf("bristolgetsid(%x, %i): %i\n", baudio, sid, tsid); */
 
 	if (baudio->sid == tsid)
 		return(++tsid);
@@ -54,11 +57,12 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 {
 	int result = 0;
 
-/*	printf("bristolSystem(%i, %i, %i) %x\n", */
-/*		msg->params.bristol.operator, */
-/*		msg->params.bristol.controller, */
-/*		msg->params.bristol.valueMSB, */
-/*		msg->params.bristol.valueLSB, audiomain); */
+	/*
+	printf("bristolSystem(%i, %i) %x\n",
+		msg->params.bristol.operator,
+		msg->params.bristol.controller,
+		(msg->params.bristol.valueMSB << 7) + msg->params.bristol.valueLSB);
+	*/
 
 	/*
 	 * We are looking for a "hello" message from the GUI, which should include
@@ -102,7 +106,8 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 			 */
 			bristolArpeggiatorInit(baudio);
 
-			baudio->sid = result = getsid(audiomain->audiolist, 0);
+			baudio->sid = result = bristolgetsid(audiomain->audiolist, 0);
+
 			if (baudio->sid == -1)
 				printf("SID failure....\n");
 
@@ -204,7 +209,7 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 					break;
 
 				baudio = audiomain->audiolist;
-/*printf("init:	%i %i\n", baudio->controlid, msg->params.bristol.from); */
+
 				while (baudio != NULL)
 				{
 					if (baudio->sid == msg->params.bristol.channel)
@@ -217,6 +222,7 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				audiomain->midiflags |= (BRISTOL_PARAMMASK & flags);
 
 				initBristolAudio(audiomain, baudio);
+
 				break;
 			}
 			case BRISTOL_EXIT_ALGO:
@@ -224,18 +230,70 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				Baudio *baudio = findBristolAudio(audiomain->audiolist,
 					msg->params.bristol.channel, 0);
 
-				if ((baudio == 0) || (baudio->mixflags == 0))
+				if ((baudio == NULL) || (baudio->mixflags == 0))
 					return(0);
 
+				/*
 				printf("exit algo: %i, %x, %x\n",
 					msg->params.bristol.channel,
 						(size_t) baudio, (size_t) baudio->mixflags);
+				*/
 
 				allNotesOff(audiomain, -1);
 
+				/*
+				 * Flag that the audio thread remove the algorithm
+				 */
 				if (baudio != NULL)
 					baudio->mixflags = (BRISTOL_HOLDDOWN | BRISTOL_REMOVE);
-				sleep(1);
+
+				/*
+				 * Check if this was the last emulator. If it was then do some
+				 * shutdown operations here.
+				 */
+				if (((audiomain->audiolist == NULL)
+					|| ((audiomain->audiolist == baudio)
+						&& (baudio->next == NULL)))
+					&& (audiomain->flags & BRISTOL_MIDI_WAIT))
+				{
+					int i = 10;
+
+					printf("empty audio list, cleaning up\n");
+
+					while (audiomain->audiolist != NULL)
+					{
+						usleep(100000);
+
+						if (audiomain->audiolist != NULL)
+							printf("waiting for audiothread cleanup\n");
+
+						if (--i == 0)
+						{
+							printf("timeout on audio thread cleanup\n");
+							break;
+						}
+					}
+
+					bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, 0);
+
+					/*
+					 * It looks like this should be called first however we do
+					 * still need the audio thread to be called by jack for the
+					 * time being: the audio thread has do clean its stuff up
+					 * before exit. When it has done its cleanup it returns to
+					 * jack immediately if the audiolist is empty so now we can
+					 * unregister the interface and clean up the rest of the
+					 * app
+					 */
+					if (audiomain->flags & BRISTOL_JACK)
+					{
+						audiomain->atReq = BRISTOL_REQSTOP;
+						bristolJackInterface(NULL);
+					}
+
+					inthandler();
+				} else
+					sleep(1);
 
 				break;
 			}
@@ -293,7 +351,6 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 					break;
 
 				baudio->lowkey = (BRISTOL_PARAMMASK & flags);
-printf("l/h: %i, %i\n", baudio->lowkey, baudio->highkey);
 
 				break;
 			}
@@ -306,7 +363,6 @@ printf("l/h: %i, %i\n", baudio->lowkey, baudio->highkey);
 					break;
 
 				baudio->highkey = (BRISTOL_PARAMMASK & flags);
-printf("l/h: %i, %i\n", baudio->lowkey, baudio->highkey);
 
 				break;
 			}
@@ -359,7 +415,7 @@ printf("l/h: %i, %i\n", baudio->lowkey, baudio->highkey);
 		 * of the playlist - we have just created it.
 		 */
 		msg->params.bristol.valueLSB = result;
-		bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, result);
+		bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, result);;
 /*			audiomain->audiolist->controlid); */
 	}
 	return(0);
