@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -21,21 +21,6 @@
  */
 
 /*
- * We are going to use this file, sonic-6, as the first review of have SMP 
- * support. Prior to this there was a lot of local storage that would prevent
- * having multiple threads running this code and it needs to be made multi-
- * entrant. This means amongst other things burying parameters in the baudio
- * structure passed on the stack however the different operators also need to
- * have this before we can have it generally deployable. There are issues here
- * though. We only have a single set of buffers for the whole baudio, not per
- * voice and if multiple cores start using them the result is non-determinate.
- *
- * This probably means the best we can target is that each baudio can be a 
- * separate thread, but not voices within one baudio.
- *
- * This should only affect the audio engine code, the init operation will only
- * have a single thread.
- *
  * Organ and string grooming envelopes affect each other. Fixed
  *
  * Organ mod and spacialisation seem too similar. Fixed
@@ -89,7 +74,7 @@ u_char operator, float value)
 			TRILOGYLCL->spaceorgan = value;
 			break;
 		case 5:
-			TRILOGYLCL->mixorgan = value * 2;
+			TRILOGYLCL->mixorgan = value * 6;
 			break;
 		case 6:
 			TRILOGYLCL->panstring = value;
@@ -98,10 +83,10 @@ u_char operator, float value)
 			TRILOGYLCL->spacestring = value;
 			break;
 		case 8:
-			TRILOGYLCL->mixstring = value;
+			TRILOGYLCL->mixstring = value * 0.2;
 			break;
 		case 9:
-			TRILOGYLCL->mixsynth = value * 0.5;
+			TRILOGYLCL->mixsynth = value * 0.25;
 			break;
 		case 10:
 			TRILOGYLCL->env2filter = (value - 0.5) * 4;
@@ -357,6 +342,15 @@ bristolVoice *voice, register float *startbuf)
 	fillFreqTable(baudio, voice, TRILOGYLCL->freqbuf,
 		audiomain->samplecount, 0);
 
+/* Organ code */
+	/* Run the organ grooming envelope */
+	audiomain->palette[1]->specs->io[0].buf = TRILOGYLCL->adsrbuf;
+	(*baudio->sound[1]).operate(
+		(audiomain->palette)[1],
+		voice,
+		(*baudio->sound[1]).param,
+		voice->locals[voice->index][1]);
+
 	if (TRILOGYLCL->mixorgan != 0)
 	{
 		bristolbzero(TRILOGYLCL->outbuf1, audiomain->segmentsize);
@@ -364,15 +358,6 @@ bristolVoice *voice, register float *startbuf)
 		bristolbzero(TRILOGYLCL->oscabuf, audiomain->segmentsize);
 		bristolbzero(TRILOGYLCL->oscbbuf, audiomain->segmentsize);
 
-/* Organ code */
-		/* Run the organ grooming envelope */
-		audiomain->palette[1]->specs->io[0].buf = TRILOGYLCL->adsrbuf;
-		(*baudio->sound[1]).operate(
-			(audiomain->palette)[1],
-			voice,
-			(*baudio->sound[1]).param,
-			voice->locals[voice->index][1]);
-
 		/*
 		 * The oscillator will take some amount of LFO for vibrato but this will
 		 * be done by passing the buffer and letting the OSC do the work separately
@@ -431,19 +416,25 @@ bristolVoice *voice, register float *startbuf)
 //printf("%f %f %f\n", TRILOGYLCL->mixorgan, TRILOGYLCL->panorgan, 
 //TRILOGYLCL->spaceorgan);
 	}
+
+	/*
+	 * The organ has a very fast release as it is only for key grooming.
+	 * The string envelope will decide on the flags.
+	 */
+	voice->flags &= ~BRISTOL_KEYDONE;
 /* Organ code done */
 
 /* Strings code */
+	/* Run the string grooming envelope */
+	audiomain->palette[1]->specs->io[0].buf = TRILOGYLCL->adsrbuf;
+	(*baudio->sound[3]).operate(
+		(audiomain->palette)[1],
+		voice,
+		(*baudio->sound[3]).param,
+		voice->locals[voice->index][3]);
+
 	if (TRILOGYLCL->mixstring != 0)
 	{
-		/* Run the string grooming envelope */
-		audiomain->palette[1]->specs->io[0].buf = TRILOGYLCL->adsrbuf;
-		(*baudio->sound[3]).operate(
-			(audiomain->palette)[1],
-			voice,
-			(*baudio->sound[3]).param,
-			voice->locals[voice->index][3]);
-
 		bristolbzero(TRILOGYLCL->outbuf1, audiomain->segmentsize);
 		bristolbzero(TRILOGYLCL->outbuf2, audiomain->segmentsize);
 		bristolbzero(TRILOGYLCL->oscabuf, audiomain->segmentsize);
@@ -497,9 +488,7 @@ bristolVoice *voice, register float *startbuf)
 			TRILOGYLCL->spacestring * TRILOGYLCL->mixstring
 				 * TRILOGYLCL->mastervolume * TRILOGYLCL->panstring,
 			baudio->rightbuf, 1.0, audiomain->samplecount);
-	} else
-		if (TRILOGYLCL->mixorgan == 0)
-			voice->flags |= BRISTOL_KEYDONE;
+	}
 /* Strings code done */
 
 	return(0);
@@ -525,11 +514,8 @@ int
 operateOneTrilogyVoice(audioMain *audiomain, Baudio *baudio,
 bristolVoice *voice, register float *startbuf)
 {
-	if (TRILOGYLCL->mixsynth == 0)
-	{
-		voice->flags |= BRISTOL_KEYDONE;
-		return(0);
-	}
+//	if (TRILOGYLCL->mixsynth == 0)
+//		return(0);
 
 	bristolbzero(TRILOGYLCL->oscabuf, audiomain->segmentsize);
 	bristolbzero(TRILOGYLCL->oscbbuf, audiomain->segmentsize);
@@ -1046,6 +1032,7 @@ bristolTrilogyODCInit(audioMain *audiomain, Baudio *baudio)
 	 * Put effects in here if needed
 	initSoundAlgo(22, 0, baudio, audiomain, baudio->effect);
 	 */
+
 	if (TRILOGYLCL->oscabuf == (float *) NULL)
 		TRILOGYLCL->oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (TRILOGYLCL->oscbbuf == (float *) NULL)
