@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -265,7 +265,7 @@ register int count, register int width)
 		if ((wtpSqr = wtp + wtpSqr) >= CS80_WAVE_SZE)
 			wtpSqr -= CS80_WAVE_SZE;
 
-		if (wtp == CS80_WAVE_SZE_M)
+		if (wtp >= CS80_WAVE_SZE_M)
 			ob[obp] += (ramp[(int) wtp] + (ramp[0] - ramp[(int) wtp])
 				* gdelta) * gain;
 		else
@@ -273,7 +273,7 @@ register int count, register int width)
 				(ramp[(int) wtp] + ((ramp[(int) wtp + 1] - ramp[(int) wtp])
 				* gdelta)) * gain;
 
-		if (wtpSqr == CS80_WAVE_SZE_M)
+		if (wtpSqr >= CS80_WAVE_SZE_M)
 			ob[obp] -= (ramp[(int) wtpSqr] + (ramp[0] - ramp[(int) wtpSqr])
 				* gdelta) * gain;
 		else
