@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -258,6 +258,28 @@ static int param(bristolOP *operator, bristolOPParams *param,
 	return(0);
 }
 
+static float scale = 0.0000000001; // 0.000000001;
+static int dngx1 = 0x67452301;
+static int dngx2 = 0xefcdab89;
+
+static void
+buffermax(float *buf, float max, int count)
+{
+	for (count--;count >= 0; count--)
+	{
+		dngx1 ^= dngx2;
+
+		if (buf[count] > max)
+			buf[count] = max;
+		else if (-buf[count] > max)
+			buf[count] = -max;
+
+//		buf[count] += dngx2 * scale;
+
+		dngx2 += dngx1;
+	}
+}
+
 /*
  * Could use some optimisation, it overloads my P450 at 16 voices.
  * May introduce restrictions on voicecount per synth.
@@ -291,6 +313,8 @@ static int operate(bristolOP *operator,
 	cgain = local->cgain;
 	egain = local->egain;
 
+	buffermax(ib, DXOP_WAVE_SZE * 4, count);
+
 	if (ib[0] > 10000)
 		ib[0] = ib[1];
 
@@ -463,7 +487,7 @@ static int operate(bristolOP *operator,
 						ob[obp] += wt[(int) wtp] * (((cgain *= release) - 1)
 							* egain);
 						 */
-						if (wtp == DXOP_WAVE_SZE_M)
+						if (wtp >= DXOP_WAVE_SZE_M)
 							ob[obp] += (wt[0] * (wtp - ((float) ((int) wtp)))
 								+ wt[(int) wtp]
 								* (1.0 - (wtp - ((float) ((int) wtp)))))
@@ -531,7 +555,7 @@ static int operate(bristolOP *operator,
 							ob[obp] += wt[(int) wtp] * (((cgain *= attack) - 1)
 								* egain);
 							 */
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -581,7 +605,7 @@ static int operate(bristolOP *operator,
 							ob[obp] += wt[(int) wtp] * (((cgain *= delta) - 1)
 								* egain);
 							 */
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -647,7 +671,7 @@ static int operate(bristolOP *operator,
 							ob[obp] += wt[(int) wtp] * (((cgain *= attack2) - 1)
 								* egain);
 							 */
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -698,7 +722,7 @@ static int operate(bristolOP *operator,
 								* egain);
 							 */
 
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -766,7 +790,7 @@ static int operate(bristolOP *operator,
 								* egain);
 							 */
 
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -816,7 +840,7 @@ static int operate(bristolOP *operator,
 								* egain);
 							 */
 
-							if (wtp == DXOP_WAVE_SZE_M)
+							if (wtp >= DXOP_WAVE_SZE_M)
 								ob[obp] += (wt[0]
 									* (wtp - ((float) ((int) wtp)))
 									+ wt[(int) wtp]
@@ -875,7 +899,7 @@ static int operate(bristolOP *operator,
 					/*
 					ob[obp] += wt[(int) wtp] * (sustain - 1) * egain;
 					 */
-					if (wtp == DXOP_WAVE_SZE_M)
+					if (wtp >= DXOP_WAVE_SZE_M)
 						ob[obp] += (wt[0]
 							* (wtp - ((float) ((int) wtp)))
 							+ wt[(int) wtp]
