@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -313,7 +313,7 @@ static int operate(bristolOP *operator,
 			wform = 0;
 
 		/* This generates the sync buffer */
-		if (wtp == EXPDCO_WAVE_SZE_M)
+		if (wtp >= EXPDCO_WAVE_SZE_M)
 			sob[obp] = (wt3[wdelta] + (wt3[0] - wt3[wdelta]) * gdelta);
 		else
 			sob[obp] = (wt3[wdelta] + (wt3[wdelta + 1] - wt3[wdelta]) * gdelta);
@@ -329,7 +329,7 @@ static int operate(bristolOP *operator,
 			gain1 = (1.0 - gain2) * gain;
 			gain2 *= gain;
 
-			if (wtp == EXPDCO_WAVE_SZE_M)
+			if (wtp >= EXPDCO_WAVE_SZE_M)
 				ob[obp] += (wt1[wdelta]
 					+ (wt1[0] - wt1[wdelta]) * gdelta)
 						* gain1
@@ -353,7 +353,7 @@ static int operate(bristolOP *operator,
 			gain1 = (1.0 - gain2) * gain;
 			gain2 *= ssg;
 
-			if (wtp == EXPDCO_WAVE_SZE_M)
+			if (wtp >= EXPDCO_WAVE_SZE_M)
 				ob[obp] += (wt1[wdelta]
 					+ (wt1[0] - wt1[wdelta]) * gdelta)
 						* gain1
@@ -372,7 +372,7 @@ static int operate(bristolOP *operator,
 			 */
 			gain1 = EXPDCO_WAVE_SZE_2 - (wform - 0.66) * EXPDCO_WAVE_SZE_3;
 
-			if (wtp > gain1)
+			if (wtp >= gain1)
 			{
 				/*
 				 * This creates a pulse width modulated square wave, with a
