@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -328,7 +328,7 @@ tanh_2(float x)
 }
  */
 
-static float scale = 0.000000001;
+static float scale = 0.0000001; // 0.000000001;
 static int dngx1 = 0x67452301;
 static int dngx2 = 0xefcdab89;
 
@@ -571,7 +571,7 @@ houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 		 * other algorithms do a premixing of this however that prevents the
 		 * filter from resonating.
 		 */
-		*ob++ += V2 * (amf + (-ay3-az4 + ay2+az3 - ay1-az2)) * mix;
+		*ob++ += V2 * (amf + (-ay3-az4 + ay2+az3 - ay1-az2) * mix) * 0.5;
 		ib++;
 	}
 
@@ -916,7 +916,7 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 		az4  = ay4;
 
 		// 1/2-sample delay for phase compensation
-		amf  = (ay4+az5 + (-ay3-+az4 + ay2+az3) * mix) *0.5;
+		amf  = (ay4+az5 + (-ay3-+az4 + ay2+az3) * mix) * 0.5;
 		az5  = ay4;
 
 		// oversampling (repeat same block) and add denormal noise
@@ -937,7 +937,7 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 		// 1/2-sample delay for phase compensation
 		// Added in 12dB and 18dB phases
-		amf  = (ay4+az5 + (-ay3-az4 + ay2+az3) * mix) *0.5;
+		amf  = (ay4+az5 + (-ay3-az4 + ay2+az3) * mix) * 0.5;
 		az5  = ay4;
 
 		*ob++ += V2 * amf;
