@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -183,7 +183,8 @@ midiControl(audioMain *audiomain, bristolMidiMsg *msg)
 		 * values into our contcontroller structure even if we have a callback.
 		 */
 
-		baudio->contcontroller[c_id] = ((float) c_val) / 127;
+		if ((c_id >= 0) && (c_id < 128))
+			baudio->contcontroller[c_id] = ((float) c_val) / 127;
 		bristolMidiToGM2(baudio->GM2values, baudio->midimap, baudio->valuemap,
 			msg);
 
@@ -411,14 +412,6 @@ static int midiSystem(audioMain *audiomain, bristolMidiMsg *msg)
 	else if (adjusted < 0)
 		adjusted = 0;
 
-#ifdef DEBUG
-	printf("midiSystem(%i, %i, %i, %i)\n",
-		msg->params.bristol.SysID,
-		msg->params.bristol.L,
-		msg->params.bristol.a,
-		msg->params.bristol.b);
-#endif
-
 	/*
 	 * First take a peek to see if this is a "SLab" message.
 	 */
@@ -485,6 +478,12 @@ static int midiSystem(audioMain *audiomain, bristolMidiMsg *msg)
 		if (baudio == (Baudio *) NULL)
 			return(0);
 
+		if (baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+			printf("bSYSEX %i %i: %f\n",
+				msg->params.bristol.operator,
+				msg->params.bristol.controller,
+				adjusted);
+
 		if (msg->params.bristol.operator < baudio->soundCount) {
 			/*
 			 * Find out if this is a float val or what? Alternatively, find
