@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -199,16 +199,40 @@ printPlayList(audioMain *audiomain)
 {
 	bristolVoice *v = audiomain->playlist;
 
-	printf("printPlayList(%x, %x)\n", (size_t) audiomain->playlist, (size_t) audiomain->playlast);
+	printf("printPlayList(%x, %x)\n",
+		(size_t) audiomain->playlist, (size_t) audiomain->playlast);
 
 	while (v != NULL)
 	{
-		printf("%x, %i, %x (%x, %x) (%x)\n", (size_t) v,
-			v->key.key, (size_t) v->flags, (size_t) v->last, (size_t) v->next,
-				v->flags);
+		printf("%08x<-%08x->%08x: %i, %i, 0x%x\n",
+			(size_t) v->last,
+			(size_t) v,
+			(size_t) v->next,
+			v->index,
+			v->key.key,
+			v->flags
+		);
 
 		v = v->next;
 	}
+
+	if ((v = audiomain->newlist) != NULL)
+	{
+		printf("printNewlist(%x)\n", (size_t) v);
+		while (v != NULL)
+		{
+			printf("%08x<-%08x->%08x: %i, %i, 0x%x\n",
+				(size_t) v->last,
+				(size_t) v,
+				(size_t) v->next,
+				v->index,
+				v->key.key,
+				v->flags
+			);
+
+			v = v->next;
+		}
+	}
 }
 
 /*
@@ -217,10 +241,11 @@ printPlayList(audioMain *audiomain)
  */
 
 int
-domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
+midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 {
-	bristolVoice *voice = audiomain->playlist, *v;
+	bristolVoice *voice, *v;
 
+//printf("mNOff ");printPlayList(audiomain);
 	/*
 	 * If we get a note off, then find that note, and configure it for done. If
 	 * we have sustain active then just set the BRISTOL_SUSTAIN flag, the key
@@ -233,7 +258,9 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 	 * the long semaphore too. Hence we just request the long one as it covers
 	 * both needs.
 	 */
-	sem_wait(&audiomain->sem_long);
+	sem_wait(audiomain->sem_long);
+
+	voice = audiomain->playlist;
 
 	while (voice != NULL)
 	{
@@ -243,10 +270,6 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 			voice = voice->next;
 			continue;
 		}
-/*
-		if (((voice->keyid == msg->params.key.key)
-				|| (voice->baudio->voicecount == 1))
-*/
 
 		/*
 		 * Check monophonic note logic. If we have monophonic note logic
@@ -259,6 +282,8 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 				|| (voice->baudio->midichannel == BRISTOL_CHAN_OMNI))
 			&& (voice->baudio->notemap.flags != 0))
 		{
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				bristolMidiPrint(msg);
 			doMNL(voice, msg->params.key.key, -1, 0);
 			voice = voice->next;
 			continue;
@@ -268,12 +293,17 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 			&& ((voice->baudio->midichannel == msg->channel)
 				|| (voice->baudio->midichannel == BRISTOL_CHAN_OMNI)))
 		{
+			/* can't print the message until we know if there is debug on */
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				bristolMidiPrint(msg);
+
 			if (voice->baudio->contcontroller[BRISTOL_CC_HOLD1] > 0)
 				voice->flags |= BRISTOL_KEYSUSTAIN;
 			else {
 				/*
-				 * Do not put the note off velocity in the key velocity. It
-				 * should be seen as a separate control parameter.
+				 * Do not put the note off velocity in the key velocity, it can
+				 * cause pops and clicks if it used to control the gain.
+				 * It should be seen as a separate control parameter.
 				 *
 				 * Took the next line out. It is basically a damaged part of
 				 * the specification - if the note off is sent with default
@@ -287,9 +317,6 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 				voice->keyoff.velocity = msg->params.key.velocity;
 				voice->flags |= BRISTOL_KEYOFF;
 
-				if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
-					bristolMidiPrint(msg);
-
 				bristolArpeggiatorNoteEvent(voice->baudio, msg);
 			}
 		}
@@ -298,9 +325,8 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 	}
 
 	/*
-	 * If we got here then we did not find the voice. Scan the newlist to make
-	 * sure this was not a spurious short event - prevents us from having notes
-	 * sticking.
+	 * Scan the newlist to make sure this was not a spurious short event,
+	 * prevents us from having notes sticking.
 	 */
 	voice = audiomain->newlist;
 	while (voice != NULL)
@@ -352,25 +378,11 @@ domidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 
 		voice = voice->next;
 	}
-	sem_post(&audiomain->sem_long);
 
-/*printPlayList(audiomain); */
-	return(0);
-}
+	sem_post(audiomain->sem_long);
 
-int
-midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
-{
-#ifdef DEBUG
-	if ((audiomain->debuglevel & MIDI_DEBUG_MASK) >= MIDI_DEBUG2)
-		printf("midiNoteOff(%i, %i)\n",
-			msg->params.key.key, msg->params.key.velocity);
-#endif
+//printf("sl: ");printPlayList(audiomain);
 
-	/*
-	 * Will eventually support multitibral notes with multi note on/off
-	 */
-	domidiNoteOff(audiomain, msg);
 	return(0);
 }
 
@@ -389,6 +401,8 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	float cFreq, dFreq, dTune = 0.0, cFreqmult = 0, mappedvelocity;
 	float cfreq, dfreq, cfreqmult;
 
+//printf("domidiNoteOn()\n");
+//printf("dmNOn: ");printPlayList(audiomain);
 	/*
 	 *	locals
 	 *	Keyid
@@ -398,12 +412,6 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	 *	dTune
 	 *	cFreqmult
 	 */
-
-#ifdef DEBUG
-	if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
-		printf("domidiNoteOn(%x, %x, %x, %i)\n", audiomain, msg, baudio, key);
-#endif
-
 	if (baudio->midiflags & BRISTOL_MIDI_DEBUG1)
 		bristolMidiPrint(msg);
 
@@ -439,11 +447,13 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	if ((baudio->voicecount == 1) && (baudio->lvoices != 0)
 		&& (baudio->notemap.flags != 0))
 	{
-		voice = audiomain->playlist;
+		if ((voice = audiomain->playlist) == NULL)
+			return(0);
 
 		while (voice->baudio != baudio)
 			if ((voice = voice->next) == NULL)
 				return(0);
+
 		doMNL(voice, key, velocity, 1);
 		return(0);
 	}
@@ -600,7 +610,7 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		voice->cfreqmult = cfreqmult;
 		voice->detune = dTune;
 
-		sem_wait(&audiomain->sem_short);
+		sem_wait(audiomain->sem_short);
 
 		/*
 		 * Short piece of critical code
@@ -622,12 +632,14 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		audiomain->newlist = voice;
 		voice->last = NULL;
 
-		sem_post(&audiomain->sem_short);
+//printf("ss: ");printPlayList(audiomain);
+
+		sem_post(audiomain->sem_short);
 
 		return(0);
 	}
 
-	sem_wait(&audiomain->sem_long);
+	sem_wait(audiomain->sem_long);
 
 	/*
 	 * We have the semaphore, search for the voice. There are several cases
@@ -720,7 +732,9 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
 			printf("voicecount exceeded\n");
 
-	sem_post(&audiomain->sem_long);
+	sem_post(audiomain->sem_long);
+
+//printf("sl: ");printPlayList(audiomain);
 
 	return(0);
 }
