@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -432,7 +432,7 @@ static int operate(bristolOP *operator,
 		 */
 		gdelta = wtp - ((float) ((int) wtp));
 
-		if (wtp > PROPHETDCO_WAVE_SZE_M)
+		if (wtp >= PROPHETDCO_WAVE_SZE_M)
 			S1 = (wt[(int) wtp] + (wt[0] - wt[(int) wtp])
 				* gdelta) * gain;
 		else
@@ -448,7 +448,7 @@ static int operate(bristolOP *operator,
 		 */
 		if (dosquare)
 		{
-			if (wtp > PROPHETDCO_WAVE_SZE_M)
+			if (wtp >= PROPHETDCO_WAVE_SZE_M)
 				S2 = (ramp[(int) wtp] + (ramp[0] - ramp[(int) wtp])
 					* gdelta) * ssg;
 			else
@@ -469,7 +469,7 @@ static int operate(bristolOP *operator,
 
 			gdelta = wtpSqr - ((float) ((int) wtpSqr));
 
-			if (wtpSqr == PROPHETDCO_WAVE_SZE_M)
+			if (wtpSqr >= PROPHETDCO_WAVE_SZE_M)
 				S2 -= (ramp[(int) wtpSqr] + (ramp[0] - ramp[(int) wtpSqr])
 					* gdelta) * ssg;
 			else
