@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -258,12 +258,13 @@ printf("Wheel %i: ct %2.2f\n", wheel, tonematrix[wheel].gain);
  * table is used by the sermon to give it the hammond generated frequencies
  */
 void
-thesermon(int samplecount, float *ctab, int sineform)
+thesermon(int samplecount, int sineform)
 {
 	register int wheel, count;
 	register float index, *source, *dest, freq;
 
 	bright = sineform == 0? 0: 1;
+
 	/*
 	 * We now have 8 waves, which are anything from sines, leading edge sines,
 	 * tris and ramps. We now need to fill all 109 tables with the number of 
@@ -1042,9 +1043,7 @@ bristolHammondGetGears(char *file, char *match, tonewheels *gearbox, int count)
 	{
 		sprintf(path, "%s/memory/profiles/%s", getenv("BRISTOL"), file);
 		if ((fd = fopen(path, "r")) == NULL)
-		{
 			return;
-		}
 	}
 
 	while (fgets(param, 256, fd) != NULL)
@@ -1097,6 +1096,8 @@ bristolHammondGetGears(char *file, char *match, tonewheels *gearbox, int count)
 /* printf("%i becomes %f\n", wheel, gearbox[wheel].freq); */
 		}
 	}
+
+	fclose(fd);
 }
 
 #ifdef NOT_USED
