@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -265,7 +265,7 @@ register int count, register float inv)
 				+ wt1[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))))
 					* gain;
 		 */
-		if ((((int) wtp) == TRILOGY_WAVE_SZE_M)
+		if ((wtp >= TRILOGY_WAVE_SZE_M)
 			|| ((lsv < 0) && (sb[obp] >= 0)))
 			ob[obp] += (wt1[0] * (wtp - ((float) ((int) wtp)))
 				+ wt1[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))))
