@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@
 #include <math.h>
 
 #include "bristol.h"
+#include "bristolblo.h"
 #include "vox.h"
 
 float note_diff;
@@ -45,7 +46,7 @@ int samplecount;
 
 static void fillWave(float *, int, int);
 static void buildVoxSound(bristolOP *, bristolOPParams *, unsigned char);
-static void fillVoxWave(bristolOP *);
+static void fillVoxWave(bristolOP *, int);
 static void fillVoxM2Wave(bristolOP *, bristolOPParams *);
 
 /*
@@ -76,7 +77,7 @@ static float sweeps[16] = {2, 4, 8, 6, 12, 16, 20, 24, 32, 0,0,0,0,0,0,0};
  */
 #define OPNAME "Vox"
 #define OPDESCRIPTION "Digitally Controlled Oscillator"
-#define PCOUNT 2
+#define PCOUNT 3
 #define IOCOUNT 2
 
 #define VOX_IN_IND 0
@@ -141,6 +142,7 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	param->param[7].int_val = 0;
 	param->param[4].float_val = 0.166880;
 	param->param[1].float_val = 1;
+	param->param[5].int_val = 0;
 	return(0);
 }
 
@@ -162,6 +164,7 @@ static int param(bristolOP *operator, bristolOPParams *param,
 
 	switch (index) {
 		case 0:
+			param->param[index].float_val = value;
 			buildVoxSound(operator, param, (int) (value * CONTROLLER_RANGE));
 			break;
 		case 1: /* gain */
@@ -180,6 +183,12 @@ static int param(bristolOP *operator, bristolOPParams *param,
 			else
 				param->param[index].int_val = 1;
 			break;
+		case 5:
+			/* Wavefore - 0=infinite, X=deformed */
+			param->param[index].int_val =  (int) (value * CONTROLLER_RANGE);;
+			buildVoxSound(operator, param,
+				(int) (param->param[0].float_val * CONTROLLER_RANGE));
+			break;
 	}
 
 	return(0);
@@ -235,7 +244,7 @@ static int operate(bristolOP *operator,
 		 * Take a sample from the wavetable into the output buffer. This 
 		 * should also be scaled by gain parameter.
 		 */
-		if (wtp == VOX_WAVE_SZE) {
+		if (wtp >= VOX_WAVE_SZE_M) {
 			ob[obp] += (wt[0] * (wtp - ((float) ((int) wtp)))
 				+ wt[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))))
 				* gain;
@@ -251,7 +260,7 @@ static int operate(bristolOP *operator,
 				* gain;
 		}
 
-		if ((wtp += ib[obp++] * 0.25) >= VOX_WAVE_SZE)
+		if ((wtp += ib[obp++] * 0.25) >= (float) VOX_WAVE_SZE)
 			wtp -= VOX_WAVE_SZE;
 	}
 
@@ -367,11 +376,24 @@ voxdcoinit(bristolOP **operator, int index, int samplerate, int samplecount)
 	return(*operator);
 }
 
+static void filltriwave(register float *mem, register int count,
+register double reach)
+{
+	register int i;
+
+	if (bloflags & BRISTOL_BLO)
+	{
+		for (i = 0;i < count; i++)
+			mem[i] = blotriangle[i];
+		return;
+	}
+}
+
 /*
  * Hm, just in case you could not tell, this is a phase distortion oscillator.
  * It is somewhat warmer than the original vox.
  */
-static void filltriwave(register float *mem, register int count,
+static void fillpdwave(register float *mem, register int count,
 register double reach)
 {
 	register int i, recalc1 = 1, recalc2 = 1;
@@ -427,29 +449,30 @@ fillWave(register float *mem, register int count, int type)
 			 * This will be a sine wave. We have count samples, and
 			 * 2PI radians in a full sine wave. Thus we take
 			 * 		(2PI * i / count) * 2048.
-			 */
 			filltriwave(mem, count, 1.0);
+			 */
+			fillpdwave(mem, count, 1.0);
 		case 1:
 		default:
-			filltriwave(mem, count, 1.05);
+			fillpdwave(mem, count, 1.05);
 			return;
 		case 2:
-			filltriwave(mem, count, 1.10);
+			fillpdwave(mem, count, 1.10);
 			return;
 		case 3:
-			filltriwave(mem, count, 1.30);
+			fillpdwave(mem, count, 1.30);
 			return;
 		case 4:
-			filltriwave(mem, count, 1.60);
+			fillpdwave(mem, count, 1.60);
 			return;
 		case 5:
-			filltriwave(mem, count, 2.0);
+			fillpdwave(mem, count, 2.0);
 			return;
 		case 6:
-			filltriwave(mem, count, 3.0);
+			fillpdwave(mem, count, 3.0);
 			return;
 		case 7:
-			filltriwave(mem, count, 10.0);
+			fillpdwave(mem, count, 10.0);
 			return;
 	}
 }
@@ -475,13 +498,13 @@ buildVoxSound(bristolOP *operator, bristolOPParams *params, unsigned char parm)
 	}
 
 	if (params->param[4].int_val == 0)
-		fillVoxWave(operator);
+		fillVoxWave(operator, params->param[5].int_val);
 	else
 		fillVoxM2Wave(operator, params);
 }
 
 static void
-fillVoxWave(bristolOP *operator)
+fillVoxWave(bristolOP *operator, int form)
 {
 	register float gain, *source, *dest, sweep, oscindex;
 	register int i, j;
@@ -490,7 +513,7 @@ fillVoxWave(bristolOP *operator)
 	specs = (bristolVOX *) operator->specs;
 
 #ifdef BRISTOL_DBG
-	printf("fillVoxWave(%x)\n", operator);
+	printf("fillVoxWave(%x, %i)\n", operator, form);
 #endif
 
 	/*
@@ -519,15 +542,20 @@ fillVoxWave(bristolOP *operator)
 	for (i = 0; i < 3; i++)
 	{
 		oscindex = 0;
+
 		/*
 		 * Take the sinewave and the destination wave.
 		 */
-		source = specs->wave[0];
+		if (form == 0) {
+			source = specs->wave[0];
+			gain = ((float) (wavelevel[i] * wavelevel[4])) / 64.0;
+		} else {
+			source = blotriangle;
+			gain = ((float) (wavelevel[i] * wavelevel[4])) / 3.0;
+		}
 
 		dest = wave1;
 
-		gain = ((float) (wavelevel[i] * wavelevel[4])) / 64.0;
-
 		if (gain == 0)
 			continue;
 
@@ -572,21 +600,25 @@ fillVoxWave(bristolOP *operator)
 	/*
 	 * Do Draw-IV for the flute, or sine waves.
 	 */
-	for (i = 4; i < 9; i++)
+	for (i = 3; i < 9; i++)
 	{
-		if (i == 7)
+		if ((i == 6) || (i == 7))
 			continue;
 
 		oscindex = 0;
 		/*
 		 * Take the sinewave and the destination wave.
 		 */
-		source = specs->wave[0];
+		if (form == 0) {
+			source = specs->wave[0];
+			gain = ((float) (wavelevel[3] * wavelevel[4])) / 64.0;
+		} else {
+			source = blotriangle;
+			gain = ((float) (wavelevel[3] * wavelevel[4])) / 3.0;
+		}
 
 		dest = wave1;
 
-		gain = ((float) (wavelevel[3] * wavelevel[4])) / 64.0;
-
 		if (gain == 0)
 			continue;
 
@@ -603,9 +635,9 @@ fillVoxWave(bristolOP *operator)
 	/*
 	 * Do Draw-IV for the reed, or ramp waves.
 	 */
-	for (i = 4; i < 9; i++)
+	for (i = 3; i < 9; i++)
 	{
-		if (i == 7)
+		if ((i == 6) || (i == 7))
 			continue;
 
 		oscindex = 0;
@@ -630,7 +662,6 @@ fillVoxWave(bristolOP *operator)
 				oscindex -= VOX_WAVE_SZE;
 		}
 	}
-
 }
 
 /*
