@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -44,6 +44,9 @@
 #include <unistd.h>
 #include <semaphore.h>
 #include <time.h>
+#include <pthread.h>
+
+#include <config.h>
 
 /* #include "bristolvers.h" */
 #include "bristoldebug.h"
@@ -158,7 +161,8 @@
 #define BRISTOL_POLY800			0x0029
 #define BRISTOL_BME700			0x002a
 #define BRISTOL_BASSMAKER		0x002b
-#define BRISTOL_SID_M1			0x002c
+#define BRISTOL_SID_M1			0x002c /* 2 SID, audio and mod - sidney */
+#define BRISTOL_SID_M2			0x002d /* 6 SID, 5 audio, 1 mod - canberra */
 
 /*
  * Operators
@@ -208,8 +212,8 @@
 #define BRISTOL_OSS				0x02000000
 #define BRISTOL_SLAB			0x04000000
 #define BRISTOL_JACK			0x08000000
-#define BRISTOL_DSSI			0x10000000 /* Unlikely to be supported */
-#define BRISTOL_LADSP			0x20000000 /* Unlikely to be supported */
+#define BRISTOL_DSSI			0x10000000 /* Unlikely to be supported - LV2? */
+#define BRISTOL_OSC				0x20000000
 #define BRISTOL_DUMMY			0x40000000
 
 #define BRISTOL_AUDIOWAIT		0x01000000
@@ -229,6 +233,7 @@
 
 #define BRISTOL_AUTO_CONN		0x00008000
 
+#define BRISTOL_TERM -3
 #define BRISTOL_FAIL -2
 #define BRISTOL_EXIT -1
 #define BRISTOL_OK 0
@@ -592,8 +597,8 @@ typedef struct AudioMain {
 	bristolVoice *newlast;
 	bristolOP **palette; /* operator templates */
 	bristolOP **effects; /* operator templates */
-	sem_t sem_long;
-	sem_t sem_short;
+	sem_t *sem_long;
+	sem_t *sem_short;
 	struct timespec abstime;
 	int voiceCount;
 	int opCount;
@@ -604,10 +609,15 @@ typedef struct AudioMain {
 	int samplecount;
 	int segmentsize;
 	int samplerate;
+	int priority;
+	struct sched_param asp;
+	struct sched_param msp;
 	int atReq;
 	int atStatus;
 	int mtReq;
 	int mtStatus;
+	int controlHandle; /* Sysex MIDI handle */
+	int midiHandle; /* MIDI handle */
 	unsigned int midiflags;
 	int s440holder; /* not used? */
 	char *audiodev; /* both of these really need to become multiheaded? */
