@@ -1,7 +1,7 @@
 
 /*
  *  Diverse SLab Include Files
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -25,15 +25,6 @@
 #include "slabrevisions.h"
 #include "slabdefinitions.h"
 
-#ifdef LINUX
-/*
- * We should take this out, it should only be needed by the libslabaudio code
- */
-#ifndef INC_SOURCE
-#include <sys/soundcard.h>
-#endif
-#endif
-
 #include "slabTimer.h"
 #ifdef NEW_DBG
 #include "slabDebugMasks.h"
