@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -85,6 +85,7 @@ configurePanel(brightonWindow *bwin, brightonIResource *panel,
 		{
 			if (event->intvalue)
 			{
+//printf("	REQ EXPOSE %x %x\n", (size_t) event, (size_t) panel);
 				panel->flags &= ~BRIGHTON_WITHDRAWN;
 
 				event->command = BRIGHTON_RESIZE;
@@ -110,6 +111,7 @@ configurePanel(brightonWindow *bwin, brightonIResource *panel,
 							(brightonDevice *) panel->devlocn[dev].dev, 0);
 				}
 			} else {
+//printf("	REQ UNEXPOSE %x %x\n", (size_t) event, (size_t) panel);
 				panel->flags |= BRIGHTON_WITHDRAWN;
 				/*
 				 * On unexpose events we dont do any redraws? Conceptually we
@@ -141,7 +143,31 @@ configurePanel(brightonWindow *bwin, brightonIResource *panel,
 		panel->canvas = brightonCreateBitmap(bwin, event->w, event->h);
 
 		if (panel->flags & BRIGHTON_WITHDRAWN)
+		{
+			/*
+			 * NOTE:
+			 * this was a fix for incorrect shadow rendering if panels are
+			 * resized whilst withdrawn.
+			 */
+			for (dev = 0; dev < panel->ndevices; dev++)
+			{
+				if (panel->devlocn[dev].type == -1)
+					continue;
+
+				event->x = (int) panel->devlocn[dev].x * panel->sw / 1000;
+				event->y = (int) panel->devlocn[dev].y * panel->sh / 1000;
+				event->w = (int) panel->devlocn[dev].width * panel->sw / 1000;
+				event->h = (int) panel->devlocn[dev].height * panel->sh / 1000;
+
+				brightonPanelLocation(bwin,
+					panel->devlocn[dev].panel, panel->devlocn[dev].index,
+					event->x, event->y, event->w, event->h);
+
+				((brightonDevice *) panel->devlocn[dev].dev)->configure(
+					panel->devlocn[dev].dev, event);
+			}
 			return(0);
+		}
 
 		/*
 		 * Render our panel canvas.
@@ -432,7 +458,6 @@ brightonCreateInterface(brightonWindow *bwin, brightonApp *app)
 
 /*	if (bwin->app->init) */
 /*		bwin->app->init(bwin); */
-
 	bwin->app->resources = (brightonIResource *)
 		brightonmalloc(bwin->app->nresources * sizeof(brightonIResource));
 
