@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -256,7 +256,10 @@ BNextEvent(brightonDisplay *display, brightonEvent *event)
 				 * This is a bit rough at the moment.
 				 */
 				if (xevent.xclient.data.l[0] == wmDeleteMessage)
+				{
+					BAutoRepeat(display, 1);
 					kill(getpid(), SIGPIPE);
+				}
 				break;
 			default:
 				break;
