@@ -138,6 +138,184 @@
 #define B_SID_TRI		1
 #define B_SID_SQUARE	2
 
+/*
+ * These were millisecond rates for the envelope attack. Decay and release
+ * were specified at 3 times all these values however I think that came more
+ * from the scaled (exponential) deltas than the actual value, ie, decay times
+ * varied by time since the attack was linear and the decay was somewhat
+ * arbitrarily exponential which needs to be studied.
+ */
+static float bSidEnvRates[16] = {
+	0.002,
+	0.006,
+	0.016,
+	0.024,
+	0.038,
+	0.056,
+	0.068,
+	0.080,
+	0.100,
+	0.250,
+	0.500,
+	0.800,
+	1.000,
+	3.000,
+	5.000,
+	8.000
+};
+
+/*
+ * The following table is actually based on a 2MHz divider circuit, it is not
+ * actually equally tempered.
+ */
+unsigned short freqmap[128] = {
+	(unsigned short) (8.175879 / B_SID_FREQ_DIV),
+	(unsigned short) (8.661983 / B_SID_FREQ_DIV),
+	(unsigned short) (9.177091 / B_SID_FREQ_DIV),
+	(unsigned short) (9.722803 / B_SID_FREQ_DIV),
+	(unsigned short) (10.300889 / B_SID_FREQ_DIV),
+	(unsigned short) (10.913456 / B_SID_FREQ_DIV),
+	(unsigned short) (11.562431 / B_SID_FREQ_DIV),
+	(unsigned short) (12.249948 / B_SID_FREQ_DIV),
+	(unsigned short) (12.978416 / B_SID_FREQ_DIV),
+	(unsigned short) (13.750051 / B_SID_FREQ_DIV),
+	(unsigned short) (14.567703 / B_SID_FREQ_DIV),
+	(unsigned short) (15.434004 / B_SID_FREQ_DIV),
+	(unsigned short) (16.351892 / B_SID_FREQ_DIV),
+	(unsigned short) (17.324118 / B_SID_FREQ_DIV),
+	(unsigned short) (18.354349 / B_SID_FREQ_DIV),
+	(unsigned short) (19.445795 / B_SID_FREQ_DIV),
+	(unsigned short) (20.601990 / B_SID_FREQ_DIV),
+	(unsigned short) (21.826912 / B_SID_FREQ_DIV),
+	(unsigned short) (23.125130 / B_SID_FREQ_DIV),
+	(unsigned short) (24.500196 / B_SID_FREQ_DIV),
+	(unsigned short) (25.957170 / B_SID_FREQ_DIV),
+	(unsigned short) (27.500481 / B_SID_FREQ_DIV),
+	(unsigned short) (29.135832 / B_SID_FREQ_DIV),
+	(unsigned short) (30.868008 / B_SID_FREQ_DIV),
+	(unsigned short) (32.704319 / B_SID_FREQ_DIV),
+	(unsigned short) (34.648834 / B_SID_FREQ_DIV),
+	(unsigned short) (36.709373 / B_SID_FREQ_DIV),
+	(unsigned short) (38.892345 / B_SID_FREQ_DIV),
+	(unsigned short) (41.204830 / B_SID_FREQ_DIV),
+	(unsigned short) (43.654778 / B_SID_FREQ_DIV),
+	(unsigned short) (46.251331 / B_SID_FREQ_DIV),
+	(unsigned short) (49.000393 / B_SID_FREQ_DIV),
+	(unsigned short) (51.915688 / B_SID_FREQ_DIV),
+	(unsigned short) (55.002476 / B_SID_FREQ_DIV),
+	(unsigned short) (58.271664 / B_SID_FREQ_DIV),
+	(unsigned short) (61.736015 / B_SID_FREQ_DIV),
+	(unsigned short) (65.410782 / B_SID_FREQ_DIV),
+	(unsigned short) (69.300072 / B_SID_FREQ_DIV),
+	(unsigned short) (73.421440 / B_SID_FREQ_DIV),
+	(unsigned short) (77.784691 / B_SID_FREQ_DIV),
+	(unsigned short) (82.413055 / B_SID_FREQ_DIV),
+	(unsigned short) (87.313370 / B_SID_FREQ_DIV),
+	(unsigned short) (92.506935 / B_SID_FREQ_DIV),
+	(unsigned short) (98.000786 / B_SID_FREQ_DIV),
+	(unsigned short) (103.831375 / B_SID_FREQ_DIV),
+	(unsigned short) (110.011002 / B_SID_FREQ_DIV),
+	(unsigned short) (116.550117 / B_SID_FREQ_DIV),
+	(unsigned short) (123.472031 / B_SID_FREQ_DIV),
+	(unsigned short) (130.821564 / B_SID_FREQ_DIV),
+	(unsigned short) (138.600143 / B_SID_FREQ_DIV),
+	(unsigned short) (146.842880 / B_SID_FREQ_DIV),
+	(unsigned short) (155.569382 / B_SID_FREQ_DIV),
+	(unsigned short) (164.826111 / B_SID_FREQ_DIV),
+	(unsigned short) (174.641983 / B_SID_FREQ_DIV),
+	(unsigned short) (185.013870 / B_SID_FREQ_DIV),
+	(unsigned short) (196.001572 / B_SID_FREQ_DIV),
+	(unsigned short) (207.684326 / B_SID_FREQ_DIV),
+	(unsigned short) (220.022003 / B_SID_FREQ_DIV),
+	(unsigned short) (233.100235 / B_SID_FREQ_DIV),
+	(unsigned short) (246.974564 / B_SID_FREQ_DIV),
+	(unsigned short) (261.643127 / B_SID_FREQ_DIV),
+	(unsigned short) (277.238708 / B_SID_FREQ_DIV),
+	(unsigned short) (293.685760 / B_SID_FREQ_DIV),
+	(unsigned short) (311.138763 / B_SID_FREQ_DIV),
+	(unsigned short) (329.706573 / B_SID_FREQ_DIV),
+	(unsigned short) (349.283966 / B_SID_FREQ_DIV),
+	(unsigned short) (370.096222 / B_SID_FREQ_DIV),
+	(unsigned short) (392.003143 / B_SID_FREQ_DIV),
+	(unsigned short) (415.454926 / B_SID_FREQ_DIV),
+	(unsigned short) (440.140839 / B_SID_FREQ_DIV),
+	(unsigned short) (466.200470 / B_SID_FREQ_DIV),
+	(unsigned short) (494.071136 / B_SID_FREQ_DIV),
+	(unsigned short) (523.286255 / B_SID_FREQ_DIV),
+	(unsigned short) (554.631165 / B_SID_FREQ_DIV),
+	(unsigned short) (587.544067 / B_SID_FREQ_DIV),
+	(unsigned short) (622.277527 / B_SID_FREQ_DIV),
+	(unsigned short) (659.630615 / B_SID_FREQ_DIV),
+	(unsigned short) (698.812012 / B_SID_FREQ_DIV),
+	(unsigned short) (740.192444 / B_SID_FREQ_DIV),
+	(unsigned short) (784.313721 / B_SID_FREQ_DIV),
+	(unsigned short) (831.255188 / B_SID_FREQ_DIV),
+	(unsigned short) (880.281677 / B_SID_FREQ_DIV),
+	(unsigned short) (932.835815 / B_SID_FREQ_DIV),
+	(unsigned short) (988.142273 / B_SID_FREQ_DIV),
+	(unsigned short) (1047.120361 / B_SID_FREQ_DIV),
+	(unsigned short) (1109.877930 / B_SID_FREQ_DIV),
+	(unsigned short) (1175.088135 / B_SID_FREQ_DIV),
+	(unsigned short) (1245.329956 / B_SID_FREQ_DIV),
+	(unsigned short) (1319.261230 / B_SID_FREQ_DIV),
+	(unsigned short) (1398.601440 / B_SID_FREQ_DIV),
+	(unsigned short) (1481.481445 / B_SID_FREQ_DIV),
+	(unsigned short) (1569.858765 / B_SID_FREQ_DIV),
+	(unsigned short) (1663.893555 / B_SID_FREQ_DIV),
+	(unsigned short) (1760.563354 / B_SID_FREQ_DIV),
+	(unsigned short) (1865.671631 / B_SID_FREQ_DIV),
+	(unsigned short) (1976.284546 / B_SID_FREQ_DIV),
+	(unsigned short) (2096.436035 / B_SID_FREQ_DIV),
+	(unsigned short) (2222.222168 / B_SID_FREQ_DIV),
+	(unsigned short) (2352.941162 / B_SID_FREQ_DIV),
+	(unsigned short) (2493.765625 / B_SID_FREQ_DIV),
+	(unsigned short) (2638.522461 / B_SID_FREQ_DIV),
+	(unsigned short) (2801.120361 / B_SID_FREQ_DIV),
+	(unsigned short) (2967.359131 / B_SID_FREQ_DIV),
+	(unsigned short) (3144.654053 / B_SID_FREQ_DIV),
+	(unsigned short) (3333.333252 / B_SID_FREQ_DIV),
+	(unsigned short) (3521.126709 / B_SID_FREQ_DIV),
+	/* This was the maximum frequency, the rest use octave foldback */
+	(unsigned short) (3731.343262 / B_SID_FREQ_DIV),
+
+	(unsigned short) (1976.284546 / B_SID_FREQ_DIV),
+	(unsigned short) (2096.436035 / B_SID_FREQ_DIV),
+	(unsigned short) (2222.222168 / B_SID_FREQ_DIV),
+	(unsigned short) (2352.941162 / B_SID_FREQ_DIV),
+	(unsigned short) (2493.765625 / B_SID_FREQ_DIV),
+	(unsigned short) (2638.522461 / B_SID_FREQ_DIV),
+	(unsigned short) (2801.120361 / B_SID_FREQ_DIV),
+	(unsigned short) (2967.359131 / B_SID_FREQ_DIV),
+	(unsigned short) (3144.654053 / B_SID_FREQ_DIV),
+	(unsigned short) (3333.333252 / B_SID_FREQ_DIV),
+	(unsigned short) (3521.126709 / B_SID_FREQ_DIV),
+	(unsigned short) (3731.343262 / B_SID_FREQ_DIV),
+
+	(unsigned short) (1976.284546 / B_SID_FREQ_DIV),
+	(unsigned short) (2096.436035 / B_SID_FREQ_DIV),
+	(unsigned short) (2222.222168 / B_SID_FREQ_DIV),
+	(unsigned short) (2352.941162 / B_SID_FREQ_DIV),
+	(unsigned short) (2493.765625 / B_SID_FREQ_DIV),
+	(unsigned short) (2638.522461 / B_SID_FREQ_DIV),
+	(unsigned short) (2801.120361 / B_SID_FREQ_DIV),
+	(unsigned short) (2967.359131 / B_SID_FREQ_DIV),
+	(unsigned short) (3144.654053 / B_SID_FREQ_DIV),
+};
+
+/*
+ * Using the following definitions we can use calls such as the following:
+ *
+ *	sid_register(id, B_SID_V1_FREQ_LO, phasemap[69][FREQ_LO]);
+ *	sid_register(id, B_SID_V1_FREQ_HI, phasemap[69][FREQ_HI]);
+ *
+ * Which would set voice zero oscillation to A_440, ie, for MIDI key 69.
+ */
+#define FREQ_LO 0
+#define FREQ_HI 1
+
+typedef unsigned char SIDPhase[2];
+static SIDPhase *phasemap = (SIDPhase *) freqmap;
+
 /*
  * Structure for the phase accumulating oscillators. There are 3 per voices
  * however in normal operation only one is used and the other waveforms are
@@ -235,11 +413,12 @@ typedef struct SidFilter {
 
 typedef struct BSid {
 	int id;
-	unsigned int nominalclock;
+	float nominalclock;
 	float samplerate;
 	float volume;
 	float gn;
 	float detune;
+	float detune2;
 	float leakage;
 	float snratio;
 	float dcbias;
@@ -406,17 +585,17 @@ bSidVoice(int id, unsigned char comm, unsigned char param)
 	SID[id]->reg[comm] = param;
 
 	if (comm >= B_SID_V3_FREQ_LO)
-		vid = 2;
+		vid = B_SID_VOICE_3;
 	else if (comm >= B_SID_V2_FREQ_LO)
-		vid = 1;
+		vid = B_SID_VOICE_2;
 
 	comm %= 7;
 
 	switch (comm) {
 		case B_SID_V1_FREQ_LO:
 		case B_SID_V1_FREQ_HI:
-			rate = (SID[id]->reg[7 * vid + B_SID_V1_FREQ_HI] << 8)
-				+ SID[id]->reg[7 * vid + B_SID_V1_FREQ_LO];
+			rate = ((SID[id]->reg[7 * vid + B_SID_V1_FREQ_HI] << 8)
+				+ SID[id]->reg[7 * vid + B_SID_V1_FREQ_LO]);
 			/*
 			 * This is the 1MHz clock phase accumulation rate. We are not
 			 * running at 1MHz, we are running at samplerate so we need to 
@@ -426,25 +605,31 @@ bSidVoice(int id, unsigned char comm, unsigned char param)
 				= (int) (rate * SID[id]->nominalclock / SID[id]->samplerate);
 
 			if (_B_SID_DEBUG)
-				printf("v %i: freq pre: %4.3f (%4.3f Hz), post: %i (%4.3f Hz) (%4.3f)\n",
+				printf("v %i: freq pre: %4.3f (%4.3f Hz), post: %i (%4.3f Hz) (d=%4.3f) (r=%5.0f)\n",
 					id,
+
 					rate,
 					rate * B_SID_FREQ_DIV,
+
 					SID[id]->voice[vid].osc[B_SID_RAMP].frequency,
-					(int)((float) SID[id]->voice[vid].osc[B_SID_RAMP].frequency)
+					SID[id]->voice[vid].osc[B_SID_RAMP].frequency
 						* SID[id]->samplerate / SID[id]->nominalclock
 						* B_SID_FREQ_DIV,
-						SID[id]->detune
+
+					SID[id]->detune,
+					SID[id]->samplerate
 				);
 
 			if (SID[id]->reg[B_SID_CONTROL] & B_SID_C_MULTI_V)
 			{
 				SID[id]->voice[vid].osc[B_SID_TRI].frequency
-					= SID[id]->voice[vid].osc[B_SID_RAMP].frequency
-						* SID[id]->detune;
+					= (int)
+						(((float) SID[id]->voice[vid].osc[B_SID_RAMP].frequency)
+						* SID[id]->detune);
 				SID[id]->voice[vid].osc[B_SID_SQUARE].frequency
-					= SID[id]->voice[vid].osc[B_SID_RAMP].frequency
-						* SID[id]->detune;
+					= (int)
+						(((float) SID[id]->voice[vid].osc[B_SID_RAMP].frequency)
+						* SID[id]->detune2);
 			} else {
 				SID[id]->voice[vid].osc[B_SID_TRI].frequency
 					= SID[id]->voice[vid].osc[B_SID_SQUARE].frequency
@@ -573,6 +758,7 @@ bSidControl(int id, unsigned char comm, unsigned char param)
 		 * and then call each register.
 		 */
 		SID[id]->detune = 1.0;
+		SID[id]->detune2 = 1.0;
 		SID[id]->leakage = 0.0001;
 		SID[id]->dcbias = -0.2;
 		SID[id]->snratio = 0.000001;
@@ -612,8 +798,8 @@ bSidControl(int id, unsigned char comm, unsigned char param)
 		 * Clean up the oscillators:
 		 *
 		 * Fout = Fp * Fclock / 16777216 Hz
-		 * Fout = 440, Fclock = 1020000 (SID[id]->nominalclock)
-		 * Fp = 440 * 16777216 / 1020000; 
+		 * Fout = 440, Fclock = 1023000 (SID[id]->nominalclock)
+		 * Fp = 440 * 16777216 / 1023000; 
 		 *
 		 * Fp = 7381 = 1CD6H
 		 *
@@ -733,18 +919,12 @@ unsigned int psample)
 {
 	int gain = 0;
 
-	if (vflags & B_SID_V_TEST) {
+	if (vflags & B_SID_V_TEST)
+	{
 		voice->osc[B_SID_TRI].current = voice->osc[B_SID_RAMP].current = 0;
 		voice->osc[B_SID_SQUARE].current = 0x0fff;
 		voice->mix = 0;
-
-		/* We can't do this, it would suppress noise on other voices
-		voice->noise.dngx1 = DNGX1;
-		voice->noise.dngx2 = DNGX2;
-		 */
-
 		voice->lsample = psample;
-
 		return;
 	}
 
@@ -1059,9 +1239,9 @@ bSidDoFilter(bSid *s)
 		s->sfMix = (s->filter.amf * V2);
 		 */
 		s->sfMix = (s->filter.amf
-			+ (-s->filter.ay3-s->filter.az4
-				+ s->filter.ay2+s->filter.az3
-				- s->filter.ay1-s->filter.az2) * s->filter.fmix) * V2 * 0.5;
+			+ (s->filter.ay3-s->filter.az3
+			- s->filter.ay2+s->filter.az4
+			- s->filter.ay1-s->filter.az2) * s->filter.fmix) * V2 * 0.5;
 	} else
 		s->sfMix = 0;
 	/* mix filter output into output buffer */
@@ -1145,6 +1325,8 @@ bSidIOAnalogue(int id, float a_in)
 		s->reg[B_SID_CONTROL], s->reg[B_SID_V3_CONTROL],
 		s->voice[B_SID_VOICE_2].mix);
 
+	s->reg[B_SID_X_ANALOGUE] = (s->voice[B_SID_VOICE_1].mix >> 4);
+	s->reg[B_SID_Y_ANALOGUE] = (s->voice[B_SID_VOICE_2].mix >> 4);
 	s->reg[B_SID_OSC_3_OUT] = (s->voice[B_SID_VOICE_3].mix >> 4);
 
 	bSidDoEnv(&(s->voice[B_SID_VOICE_3]), s->reg[B_SID_V3_CONTROL]);
@@ -1217,6 +1399,7 @@ bSidIODetune(int id, float param)
 		printf("s %i: DETUNE %f cents\n", id, param);
 
 	SID[id]->detune = powf(SID[id]->cent_diff, param);
+	SID[id]->detune2 = 1 / SID[id]->detune;
 
 	return(SID[id]->detune);
 }
@@ -1253,6 +1436,27 @@ bSidIOOberheim(int id, float param)
 	return(SID[id]->filter.fmix = param * param);
 }
 
+static float
+bSidIOClockrate(int id, float param)
+{
+	if (_B_SID_DEBUG_A)
+		printf("s %i: Clock %f\n", id, param);
+
+	if ((param > 0.5) || (param < 3))
+	{
+		SID[id]->nominalclock = param * 1000000;
+		/* And reload the frequency registers from current value to retune */
+		sid_register(id, B_SID_V1_FREQ_LO, SID[id]->reg[B_SID_V1_FREQ_LO]);
+		sid_register(id, B_SID_V1_FREQ_HI, SID[id]->reg[B_SID_V1_FREQ_HI]);
+		sid_register(id, B_SID_V2_FREQ_LO, SID[id]->reg[B_SID_V2_FREQ_LO]);
+		sid_register(id, B_SID_V2_FREQ_HI, SID[id]->reg[B_SID_V2_FREQ_HI]);
+		sid_register(id, B_SID_V3_FREQ_LO, SID[id]->reg[B_SID_V3_FREQ_LO]);
+		sid_register(id, B_SID_V3_FREQ_HI, SID[id]->reg[B_SID_V3_FREQ_HI]);
+	}
+
+	return(SID[id]->nominalclock);
+}
+
 static float
 bSidIOSNRatio(int id, float param)
 {
@@ -1272,7 +1476,7 @@ bSidIODestroy(int id, float samplerate)
 	tmp = SID[id];
 	SID[id] = NULL;
 
-	free(tmp);
+	bristolfree(tmp);
 
 	return(0);
 }
@@ -1289,12 +1493,12 @@ bSidIOInit(int id, float samplerate)
 	if (sidID == B_SID_COUNT)
 		return(-1);
 
-	SID[sidID] = (bSid *) malloc((size_t) sizeof(bSid));
+	SID[sidID] = (bSid *) bristolmalloc((size_t) sizeof(bSid));
 	bzero((void *) SID[sidID], (size_t) sizeof(bSid));
 
 	SID[sidID]->id = sidID;
 	SID[sidID]->samplerate = samplerate;
-	SID[sidID]->nominalclock = 1020000;
+	SID[sidID]->nominalclock = B_SID_MASTER_CLOCK;
 	SID[sidID]->detune = 1.0;
 	SID[sidID]->leakage = 0.01;
 	SID[sidID]->snratio = 0.000001;
@@ -1347,6 +1551,7 @@ bSidIOInit(int id, float samplerate)
 	bSidIODispatch[B_SID_SN_LEAKAGE] = bSidIOLeakage;
 	bSidIODispatch[B_SID_DC_BIAS] = bSidIODCBias;
 	bSidIODispatch[B_SID_OBERHEIM] = bSidIOOberheim;
+	bSidIODispatch[B_SID_CLOCKRATE] = bSidIOClockrate;
 
 	/*
 	 * We continue with a B_SID_C_RESET call although this should be made by
@@ -1364,14 +1569,14 @@ bSidIOInit(int id, float samplerate)
 int
 sid_register(int id, unsigned char address, unsigned char value)
 {
-	if (_B_SID_DEBUG)
-		printf("id %x: address 0x%02x(%02i) := 0x%02x\n",
-			id, address, address, value);
-
 	if ((id >= B_SID_COUNT) || (id < 0) || (SID[id] == NULL)
 		|| (address >= B_SID_REGISTERS))
 		return(0xff);
 
+	if (_B_SID_DEBUG)
+		printf("id %x: address 0x%02x(%02i) := 0x%02x\n",
+			id, address, address, value);
+
 	return(bSidDispatch[address](id, address, value));
 }
 
