@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -69,6 +69,8 @@ extern int bristolPhysWrite();
 
 extern int initMidiLib();
 
+static char *devname = "localhost";
+
 /*
  * This is the generic code. Calls the desired (configured) drivers.
  */
@@ -82,6 +84,9 @@ void *param)
 	printf("bristolMidiOpen(%s, %x)\n", dev, flags);
 #endif
 
+	if (dev == NULL)
+		dev = devname;
+
 	initMidiLib(flags);
 
 	/*
@@ -191,6 +196,7 @@ void *param)
 	bmidi.dev[devnum].handleCount = 1;
 	bmidi.dev[devnum].bufindex = 0;
 	bmidi.dev[devnum].bufcount = 0;
+	bmidi.dev[devnum].sequence = 0;
 	bmidi.handle[handle].dev = devnum;
 	bmidi.handle[handle].callback = callback;
 	bmidi.handle[handle].param = param;
@@ -199,6 +205,36 @@ void *param)
 	return(handle);
 }
 
+int
+bristolMidiOption(int handle, int option, int value)
+{
+	if (bristolMidiSanity(handle) < 0)
+		return(bristolMidiSanity(handle));
+
+	switch (option) {
+		case BRISTOL_NRP_DEBUG:
+		{
+			int i;
+
+			if (value == 1)
+			{
+				for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
+					if (bmidi.handle[i].dev >= 0)
+						bmidi.dev[bmidi.handle[i].dev].flags
+							|= _BRISTOL_MIDI_DEBUG;
+			} else {
+				for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
+					if (bmidi.handle[i].dev >= 0)
+						bmidi.dev[bmidi.handle[i].dev].flags
+							&= ~_BRISTOL_MIDI_DEBUG;
+			}
+			break;
+		}
+	}
+
+	return(0);
+}
+
 int
 bristolMidiClose(int handle)
 {
@@ -232,6 +268,37 @@ bristolMidiClose(int handle)
 	return(BRISTOL_MIDI_DRIVER);
 }
 
+/*
+ * We need to have the ability to have a duplicated handle, take a handle
+ * that does not have a callback and give it one. This is a GUI operation
+ * only, it allows us to have a non-blocking request to the control handle
+ * for event redistribution.
+ */
+int
+bristolMidiDupFD(int handle, int flags, int (*callback)(), void *param)
+{
+	int nh, nd;
+
+	if ((nh = bristolMidiFindFreeHandle()) < 0)
+		return(nh);
+	if ((nd = bristolMidiFindDev(NULL)) < 0)
+		return(nd);
+
+	bcopy(&bmidi.handle[handle], &bmidi.handle[nh],
+		sizeof(bristolMidiHandle));
+	bcopy(&bmidi.dev[bmidi.handle[handle].dev], &bmidi.dev[nd],
+		sizeof(bristolMidiDev));
+
+	bmidi.handle[nh].messagemask = flags;
+	bmidi.handle[nh].callback = callback;
+	bmidi.handle[nh].param = param;
+	bmidi.handle[nh].dev = nd;
+
+	bmidi.dev[nd].flags |= BRISTOL_CONN_NBLOCK;
+
+	return(nh);
+}
+
 int
 bristolMidiDevRead(int dev, bristolMidiMsg *msg)
 {
@@ -261,7 +328,7 @@ bristolMidiDevRead(int dev, bristolMidiMsg *msg)
 int
 bristolGetMidiFD(int handle)
 {
-	return(bmidi.dev[bmidi.handle[handle].dev].fd);
+	return(bmidi.handle[handle].dev);
 }
 
 int
@@ -340,6 +407,31 @@ bristolMidiTerminate()
 	return(0);
 }
 
+int
+bristolMidiRawWrite(int dev, bristolMidiMsg *msg, int size)
+{
+	if (bristolMidiDevSanity(dev) < 0)
+		return(bristolMidiDevSanity(dev));
+
+#ifdef DEBUG
+	printf("bristolMidiRawWrite %i/%i, %i\n", dev, bmidi.dev[dev].fd, size);
+#endif
+printf("bristolMidiRawWrite (%x) %x/%x/%x\n",
+msg->command,
+msg->channel,
+msg->params.key.key,
+msg->params.key.velocity);
+
+	if (bristolPhysWrite(bmidi.dev[dev].fd,
+		(unsigned char *) &msg->command, 1) != 0)
+		return(1);
+	if (bristolPhysWrite(bmidi.dev[dev].fd,
+		(unsigned char *) &msg->params, size - 1) != 0)
+		return(1);
+
+	return(0);
+}
+
 int
 bristolMidiWrite(int dev, bristolMsg *msg, int size)
 {
@@ -426,9 +518,9 @@ bristolKeyEvent(int handle, int op, int channel, int key, int velocity)
 	if (bristolMidiSanity(handle) < 0)
 		return(bristolMidiSanity(handle));
 
-	if (op == BRISTOL_EVENT_KEYON) {
+	if (op == BRISTOL_EVENT_KEYON)
 		comm = MIDI_NOTE_ON | channel;
-	} else
+	else
 		comm = MIDI_NOTE_OFF | channel;
 
 	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
@@ -541,7 +633,7 @@ bristolMidiSendMsg(int handle, int channel, int operator, int controller,
 	/*
 	 * We are going to send this through to the control port as a basic format
 	 * message. The parameter is not going to be scaled, just given as a value
-	 * from 0 to 127. The receiver can decide how to scale the value.
+	 * from 0 to 16383. The receiver can decide how to scale the value.
 	 */
 	msg.SysID = 83;
 	msg.L = 76;
