@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2008
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 
 #include "bristolmidi.h"
 
-/* #define DEBUG */
+//#define DEBUG
 
 extern bristolMidiMain bmidi;
 extern int bristolFreeHandle();
@@ -54,8 +54,10 @@ int (*callback)(), void *param, int dev, int handle)
 	snd_seq_addr_t addr;
   
 #ifdef DEBUG
-	printf("bristolMidiSeqOpen(%s)\n", devname);
+	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		printf("bristolMidiSeqOpen(%s)\n", devname);
 #endif
+
 #if (SND_LIB_MAJOR == 1)
 	/*
 	 * Well, we got here, so I could perhaps consider assuming that we have
@@ -119,11 +121,15 @@ int (*callback)(), void *param, int dev, int handle)
 	snd_seq_port_subscribe_alloca(&sub);
 	addr.client = SND_SEQ_CLIENT_SYSTEM;
 	addr.port = SND_SEQ_PORT_SYSTEM_ANNOUNCE;
-printf("Registering %i %i\n", addr.client, addr.port);
+
+	//printf("Registering %i %i\n", addr.client, addr.port);
+
 	snd_seq_port_subscribe_set_sender(sub, &addr);
 	addr.client = client;
 	addr.port = port;
-printf("Registered %i %i\n", addr.client, addr.port);
+
+	printf("Registered %i %i\n", addr.client, addr.port);
+
 	snd_seq_port_subscribe_set_dest(sub, &addr);
 	snd_seq_port_subscribe_set_queue(sub, queue);
 	snd_seq_port_subscribe_set_time_update(sub, 1);
@@ -326,38 +332,35 @@ static char *event_names[256] = {
 */
 
 int
-translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
+translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 {
 #ifdef DEBUG
-	char *space = "	 ";
+	if ((bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		&& (ev->type != SND_SEQ_EVENT_CLOCK)) /* Very chatty, naturally */
+	{
+		char *space = "	 ";
 
-	printf("EVENT>>> Type = %d, flags = 0x%x", ev->type, ev->flags);
+		printf("\nEVENT>>> Type = %d, flags = 0x%x", ev->type, ev->flags);
 
-	switch (ev->flags & SND_SEQ_TIME_STAMP_MASK) {
-		case SND_SEQ_TIME_STAMP_TICK:
-			printf(", time = %d ticks", ev->time.tick);
-			break;
-		case SND_SEQ_TIME_STAMP_REAL:
-			printf(", time = %d.%09d",
-				(int)ev->time.time.tv_sec,
-				(int)ev->time.time.tv_nsec);
-			break;
+		switch (ev->flags & SND_SEQ_TIME_STAMP_MASK) {
+			case SND_SEQ_TIME_STAMP_TICK:
+				printf(", time = %d ticks", ev->time.tick);
+				break;
+			case SND_SEQ_TIME_STAMP_REAL:
+				printf(", time = %d.%09d",
+					(int)ev->time.time.tv_sec,
+					(int)ev->time.time.tv_nsec);
+				break;
+		}
+
+		printf("\n%sSource = %d.%d, dest = %d.%d, queue = %d\n",
+			space,
+			ev->source.client,
+			ev->source.port,
+			ev->dest.client,
+			ev->dest.port,
+			ev->queue);
 	}
-
-	printf("\n%sSource = %d.%d, dest = %d.%d, queue = %d\n",
-		space,
-		ev->source.client,
-		ev->source.port,
-		ev->dest.client,
-		ev->dest.port,
-		ev->queue);
-
-/*
-	if (event_names[ev->type])
-		printf("%sEvent = %s", space, event_names[ev->type]);
-	else
-		printf("%sEvent = Reserved %d\n", space, ev->type);
-*/
 #endif
 
 	msg->command = 0xff;
@@ -369,74 +372,86 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 	switch (ev->type) {
 		case SND_SEQ_EVENT_NOTE:
 #ifdef DEBUG
-			printf(
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf(
 				"; ch=%d, note=%d, velocity=%d, off_velocity=%d, duration=%d\n",
-				ev->data.note.channel,
-				ev->data.note.note,
-				ev->data.note.velocity,
-				ev->data.note.off_velocity,
-				ev->data.note.duration);
+					ev->data.note.channel,
+					ev->data.note.note,
+					ev->data.note.velocity,
+					ev->data.note.off_velocity,
+					ev->data.note.duration);
 #endif
 			break;
 
 		case SND_SEQ_EVENT_NOTEON:
-#ifdef DEBUG
-			printf("; ch=%d, note=%d, velocity=%d\n",
-				ev->data.note.channel,
-				ev->data.note.note,
-				ev->data.note.velocity);
-#endif
+			if (dev)
+				printf("c%i-%02x/%02x/%02x ",
+					dev,
+					ev->data.note.channel,
+					ev->data.note.note,
+					ev->data.note.velocity);
+
 			if (ev->data.note.velocity == 0) {
-				msg->command = MIDI_NOTE_OFF;
+				msg->command = MIDI_NOTE_OFF|ev->data.note.channel;
 				msg->params.key.velocity = 64;
 			} else {
-				msg->command = MIDI_NOTE_ON;
+				msg->command = MIDI_NOTE_ON|ev->data.note.channel;
 				msg->params.key.velocity = ev->data.note.velocity;
 			}
 			msg->channel = ev->data.note.channel;
 			msg->params.key.key = ev->data.note.note;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 3;
 			break;
 
 		case SND_SEQ_EVENT_NOTEOFF:
-#ifdef DEBUG
-			printf("; ch=%d, note=%d, velocity=%d\n",
-				ev->data.note.channel,
-				ev->data.note.note,
-				ev->data.note.velocity);
-#endif
-			msg->command = MIDI_NOTE_OFF;
+			if (dev)
+				printf("c%i-%02x/%02x/%02x ",
+					dev,
+					ev->data.note.channel,
+					ev->data.note.note,
+					ev->data.note.velocity);
+
+			msg->command = MIDI_NOTE_OFF|ev->data.note.channel;
 			msg->channel = ev->data.note.channel;
 			msg->params.key.key = ev->data.note.note,
 			msg->params.key.velocity = ev->data.note.velocity;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 3;
 			break;
 
 		case SND_SEQ_EVENT_KEYPRESS:
 #ifdef DEBUG
-			printf("; ch=%d, note=%d, velocity=%d\n",
-				ev->data.note.channel,
-				ev->data.note.note,
-				ev->data.note.velocity);
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; ch=%d, note=%d, velocity=%d\n",
+					ev->data.note.channel,
+					ev->data.note.note,
+					ev->data.note.velocity);
 #endif
-			msg->command = MIDI_POLY_PRESS;
+			msg->command = MIDI_POLY_PRESS|ev->data.note.channel;
 			msg->channel = ev->data.note.channel;
 			msg->params.pressure.key = ev->data.note.note,
 			msg->params.pressure.pressure = ev->data.note.velocity;
+			msg->params.bristol.msgLen = 3;
 			break;
 
 		case SND_SEQ_EVENT_CONTROLLER:
-#ifdef DEBUG
-			printf("; ch=%d, param=%i, value=%i\n",
-				ev->data.control.channel,
-				ev->data.control.param,
-				ev->data.control.value);
-#endif
+			if (dev)
+				printf("c%i-%02x/%02x/%02x ",
+					dev,
+					ev->data.control.channel,
+					ev->data.control.param,
+					ev->data.control.value);
+
 			/*
 			 * Hm, we may need to recode these to 7 bit midi values?
 			 */
-			msg->command = MIDI_CONTROL;
+			msg->command = MIDI_CONTROL|ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			msg->params.controller.c_id = ev->data.control.param,
 			msg->params.controller.c_val = ev->data.control.value;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 3;
 
 			/*
 			 * We should also stuff the GM-2 table for defined controller 
@@ -449,49 +464,58 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 			/*
 			 * May need to have a registered callback here from GUI.
 			 */
-#ifdef DEBUG
-			printf("; ch=%d, program=%i\n",
-				ev->data.control.channel,
-				ev->data.control.value);
-#endif
+			if (dev)
+				printf("c%i-%02x/%02x ",
+					dev,
+					ev->data.control.channel,
+					ev->data.control.value);
 
-			msg->command = MIDI_PROGRAM;
+
+			msg->command = MIDI_PROGRAM|ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			msg->params.program.p_id = ev->data.control.value;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 2;
 
 			break;
 
 		case SND_SEQ_EVENT_CHANPRESS:
-#ifdef DEBUG
-			printf("; ch=%d, value=%i\n",
-				ev->data.control.channel,
-				ev->data.control.value);
-#endif
-			msg->command = MIDI_CONTROL;
+			if (dev)
+				printf("c%i-%02x/%02x ",
+					dev,
+					ev->data.control.channel,
+					ev->data.control.value);
+
+			msg->command = MIDI_CONTROL|ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			msg->params.channelpress.pressure = ev->data.control.value;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 2;
 			break;
 
 		case SND_SEQ_EVENT_PITCHBEND:
-#ifdef DEBUG
-			printf("; ch=%d, value=%i\n",
-				ev->data.control.channel,
-				ev->data.control.value);
-#endif
+			if (dev)
+				printf("c%i-%02x/%02x ",
+					dev,
+					ev->data.control.channel,
+					ev->data.control.value);
 
 			/*
 			 * Hm, we may need to recode these to 7 bit midi values?
 			 */
-			msg->command = MIDI_PITCHWHEEL;
+			msg->command = MIDI_PITCHWHEEL|ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			ev->data.control.value += 8192;
 			msg->params.pitch.lsb = ev->data.control.value & 0x7f;
 			msg->params.pitch.msb = ev->data.control.value >> 7;
+			msg->sequence = bmidi.dev[dev].sequence++;
+			msg->params.bristol.msgLen = 3;
 			break;
 
 		case SND_SEQ_EVENT_SYSEX:
-			{
 #ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+			{
 				unsigned char *sysex =
 					(unsigned char *) ev + sizeof(snd_seq_event_t);
 				unsigned int c;
@@ -503,16 +527,17 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 						sysex[c], c < ev->data.ext.len - 1 ? ":" : "");
 				}
 				printf("]\n");
-#endif
-				/*
-				 * Hm, we should at least see if this is a SLab message?
-				 */
 			}
+#endif
+			/*
+			 * Hm, we should at least see if this is a SLab message?
+			 */
 			break;
 
 		case SND_SEQ_EVENT_QFRAME:
 #ifdef DEBUG
-			printf("; frame=0x%02x\n", ev->data.control.value);
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; frame=0x%02x\n", ev->data.control.value);
 #endif
 			break;
 
@@ -520,8 +545,9 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 		case SND_SEQ_EVENT_START:
 		case SND_SEQ_EVENT_CONTINUE:
 		case SND_SEQ_EVENT_STOP:
-#ifdef DEBUG
-			printf("; queue = %i\n", ev->data.queue.queue);
+#ifdef DONT_DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; queue = %i\n", ev->data.queue.queue);
 #endif
 			break;
 
@@ -532,22 +558,24 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 			break;
 
 		case SND_SEQ_EVENT_ECHO:
-			{
 #ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+			{
 				int i;
 				printf("; ");
 				for (i = 0; i < 8; i++) {
 					printf("%02i%s", ev->data.raw8.d[i], i < 7 ? ":" : "\n");
 				}
-#endif
 			}
+#endif
 			break;
 
 		case SND_SEQ_EVENT_CLIENT_START:
 		case SND_SEQ_EVENT_CLIENT_EXIT:
 		case SND_SEQ_EVENT_CLIENT_CHANGE:
 #ifdef DEBUG
-			printf("; client=%i\n", ev->data.addr.client);
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; client=%i\n", ev->data.addr.client);
 #endif
 			break;
 
@@ -555,17 +583,21 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg)
 		case SND_SEQ_EVENT_PORT_EXIT:
 		case SND_SEQ_EVENT_PORT_CHANGE:
 #ifdef DEBUG
-			printf("; client=%i, port = %i\n",
-				ev->data.addr.client, ev->data.addr.port);
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; client=%i, port = %i\n",
+					ev->data.addr.client, ev->data.addr.port);
 #endif
 			break;
 
 		case SND_SEQ_EVENT_PORT_SUBSCRIBED:
 		case SND_SEQ_EVENT_PORT_UNSUBSCRIBED:
 #ifdef DEBUG
-			printf("; %i:%i -> %i:%i\n",
-				ev->data.connect.sender.client, ev->data.connect.sender.port,
-				ev->data.connect.dest.client, ev->data.connect.dest.port);
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("; %i:%i -> %i:%i\n",
+					ev->data.connect.sender.client,
+					ev->data.connect.sender.port,
+					ev->data.connect.dest.client,
+					ev->data.connect.dest.port);
 #endif
 			break;
 
@@ -609,7 +641,8 @@ bristolMidiSeqRead(int dev, bristolMidiMsg *msg)
 	while ((res = snd_seq_event_input(bmidi.dev[dev].driver.seq.handle, &ev))
 		> 0)
 	{
-		translate_event(ev, msg);
+		translate_event(ev, msg,
+			bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG? dev: 0);
 
 		if (msg->command != 0xff)
 		{
